@@ -44,7 +44,7 @@
 #include <glib.h>
 #include <libxml/xmlmemory.h>
 #include <glib/gi18n-lib.h>
-#include <camel/camel-internet-address.h>
+#include <camel/camel.h>
 
 #define d(x)
 
@@ -751,7 +751,7 @@ e_destination_get_name (const EDestination *dest)
 				priv->name = g_strdup (camel_name);
 			}
 
-			camel_object_unref (CAMEL_OBJECT (addr));
+			g_object_unref (addr);
 		}
 	}
 
@@ -826,7 +826,7 @@ e_destination_get_email (const EDestination *dest)
 				priv->email = g_strdup (camel_email);
 			}
 
-			camel_object_unref (CAMEL_OBJECT (addr));
+			g_object_unref (addr);
 		}
 
 		/* Force e-mail to be non-null... */
@@ -842,12 +842,10 @@ e_destination_get_email (const EDestination *dest)
  * e_destination_get_address:
  * @dest: an #EDestination
  *
- * Gets the encoded name and email address, or in the case of lists, the
- * encoded list of email addresses, from @dest.  The returned string is
- * suitable for use in an email header, but not for displaying to users.
+ * Gets the formatted name and e-mail address, or in the case of
+ * lists, the formatted list of e-mail addresses, from @dest.
  *
- * Returns: an encoded destination string suitable for use in an
- *          email header, or %NULL if the destination was empty
+ * Returns: A formatted destination string, or %NULL if the destination was empty.
  **/
 const gchar *
 e_destination_get_address (const EDestination *dest)
@@ -905,7 +903,7 @@ e_destination_get_address (const EDestination *dest)
 			priv->addr = camel_address_encode (CAMEL_ADDRESS (addr));
 		}
 
-		camel_object_unref (CAMEL_OBJECT (addr));
+		g_object_unref (addr);
 	}
 
 	return priv->addr;
@@ -967,7 +965,7 @@ e_destination_get_textrep (const EDestination *dest, gboolean include_email)
 		camel_internet_address_add (addr, name, email);
 		g_free (dest->priv->textrep);
 		dest->priv->textrep = camel_address_format (CAMEL_ADDRESS (addr));
-		camel_object_unref (CAMEL_OBJECT (addr));
+		g_object_unref (addr);
 	}
 
 	if (dest->priv->textrep != NULL)
