@@ -19,39 +19,62 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef _CAMEL_FOLDER_SUMMARY_H
-#define _CAMEL_FOLDER_SUMMARY_H
+#if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
+#error "Only <camel/camel.h> can be included directly."
+#endif
+
+#ifndef CAMEL_FOLDER_SUMMARY_H
+#define CAMEL_FOLDER_SUMMARY_H
 
 #include <stdio.h>
 #include <time.h>
+
+#include <camel/camel-mime-message.h>
 #include <camel/camel-mime-parser.h>
-#include <camel/camel-object.h>
 #include <camel/camel-index.h>
 
-#define CAMEL_FOLDER_SUMMARY_TYPE         camel_folder_summary_get_type ()
-#define CAMEL_FOLDER_SUMMARY(obj)         CAMEL_CHECK_CAST (obj, camel_folder_summary_get_type (), CamelFolderSummary)
-#define CAMEL_FOLDER_SUMMARY_CLASS(klass) CAMEL_CHECK_CLASS_CAST (klass, camel_folder_summary_get_type (), CamelFolderSummaryClass)
-#define CAMEL_IS_FOLDER_SUMMARY(obj)      CAMEL_CHECK_TYPE (obj, camel_folder_summary_get_type ())
+/* Standard GObject macros */
+#define CAMEL_TYPE_FOLDER_SUMMARY \
+	(camel_folder_summary_get_type ())
+#define CAMEL_FOLDER_SUMMARY(obj) \
+	(G_TYPE_CHECK_INSTANCE_CAST \
+	((obj), CAMEL_TYPE_FOLDER_SUMMARY, CamelFolderSummary))
+#define CAMEL_FOLDER_SUMMARY_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_CAST \
+	((cls), CAMEL_TYPE_FOLDER_SUMMARY, CamelFolderSummaryClass))
+#define CAMEL_IS_FOLDER_SUMMARY(obj) \
+	(G_TYPE_CHECK_INSTANCE_TYPE \
+	((obj), CAMEL_TYPE_FOLDER_SUMMARY))
+#define CAMEL_IS_FOLDER_SUMMARY_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_TYPE \
+	((cls), CAMEL_TYPE_FOLDER_SUMMARY))
+#define CAMEL_FOLDER_SUMMARY_GET_CLASS(obj) \
+	(G_TYPE_INSTANCE_GET_CLASS \
+	((obj), CAMEL_TYPE_FOLDER_SUMMARY, CamelFolderSummaryClass))
 
 G_BEGIN_DECLS
 
 struct _CamelFolder;
+struct _CamelStore;
 
-/*typedef struct _CamelFolderSummary      CamelFolderSummary;*/
+typedef struct _CamelFolderSummary CamelFolderSummary;
 typedef struct _CamelFolderSummaryClass CamelFolderSummaryClass;
+typedef struct _CamelFolderSummaryPrivate CamelFolderSummaryPrivate;
 
 typedef struct _CamelMessageInfo CamelMessageInfo;
 typedef struct _CamelMessageInfoBase CamelMessageInfoBase;
 
 typedef struct _CamelFolderMetaSummary CamelFolderMetaSummary;
 
+typedef struct _CamelMessageContentInfo CamelMessageContentInfo;
+
 /* A tree of message content info structures
    describe the content structure of the message (if it has any) */
 struct _CamelMessageContentInfo {
-	struct _CamelMessageContentInfo *next;
+	CamelMessageContentInfo *next;
 
-	struct _CamelMessageContentInfo *childs;
-	struct _CamelMessageContentInfo *parent;
+	CamelMessageContentInfo *childs;
+	CamelMessageContentInfo *parent;
 
 	CamelContentType *type;
 	gchar *id;
@@ -206,10 +229,17 @@ typedef enum _CamelFolderSummaryFlags {
 	CAMEL_SUMMARY_DIRTY = 1<<0
 } CamelFolderSummaryFlags;
 
+typedef enum _CamelFolderSummaryLock {
+	CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK,
+	CAMEL_FOLDER_SUMMARY_IO_LOCK,
+	CAMEL_FOLDER_SUMMARY_FILTER_LOCK,
+	CAMEL_FOLDER_SUMMARY_ALLOC_LOCK,
+	CAMEL_FOLDER_SUMMARY_REF_LOCK
+} CamelFolderSummaryLock;
+
 struct _CamelFolderSummary {
 	CamelObject parent;
-
-	struct _CamelFolderSummaryPrivate *priv;
+	CamelFolderSummaryPrivate *priv;
 
 	/* header info */
 	guint32 version;	/* version of file loaded/loading */
@@ -223,10 +253,6 @@ struct _CamelFolderSummary {
 	guint32 junk_not_deleted_count;
 	guint32 visible_count;
 
-	/* sizes of memory objects */
-	guint32 message_info_size;
-	guint32 content_info_size;
-
 	/* memory allocators (setup automatically) */
 	struct _EMemChunk *message_info_chunks;
 	struct _EMemChunk *content_info_chunks;
@@ -234,11 +260,6 @@ struct _CamelFolderSummary {
 	gchar *summary_path;
 	gboolean build_content;	/* do we try and parse/index the content, or not? */
 
-#if 0  /* Deprecated */
-	GPtrArray *messages;	/* CamelMessageInfo's */
-	GHashTable *messages_uid; /* CamelMessageInfo's by uid */
-#endif
-
 	/* New members to replace the above depreacted members */
 	GPtrArray *uids;
 	GHashTable *loaded_infos;
@@ -261,6 +282,10 @@ struct _CamelFIRecord;
 struct _CamelFolderSummaryClass {
 	CamelObjectClass parent_class;
 
+	/* sizes of memory objects */
+	gsize message_info_size;
+	gsize content_info_size;
+
 	/* load/save the global info */
 	gint (*summary_header_load)(CamelFolderSummary *, FILE *);
 	gint (*summary_header_save)(CamelFolderSummary *, FILE *);
@@ -324,7 +349,7 @@ struct _CamelFolderMetaSummary {
 	gchar *path;		/* Path to meta-summary-file */
 };
 
-CamelType			 camel_folder_summary_get_type	(void);
+GType			 camel_folder_summary_get_type	(void);
 CamelFolderSummary      *camel_folder_summary_new	(struct _CamelFolder *folder);
 
 /* Deprecated */
@@ -347,7 +372,7 @@ gint camel_folder_summary_load_from_db (CamelFolderSummary *s, CamelException *e
 
 /* only load the header */
 gint camel_folder_summary_header_load(CamelFolderSummary *summary);
-gint camel_folder_summary_header_load_from_db (CamelFolderSummary *s, CamelStore *store, const gchar *folder_name, CamelException *ex);
+gint camel_folder_summary_header_load_from_db (CamelFolderSummary *s, struct _CamelStore *store, const gchar *folder_name, CamelException *ex);
 gint camel_folder_summary_header_save_to_db (CamelFolderSummary *s, CamelException *ex);
 
 /* set the dirty bit on the summary */
@@ -361,12 +386,8 @@ CamelMessageInfo * camel_folder_summary_peek_info (CamelFolderSummary *s, const
 
 /* Get only the uids of dirty/changed things to sync to server/db */
 GPtrArray * camel_folder_summary_get_changed (CamelFolderSummary *s);
-/* Gets the size of loaded mi's */
-gint camel_folder_summary_cache_size (CamelFolderSummary *s);
 /* reload the summary at any required point if required */
-gint camel_folder_summary_reload_from_db (CamelFolderSummary *s, CamelException *ex);
-/* ensures all CamelMessagesInfos loaded in memory */
-void camel_folder_summary_ensure_infos_loaded (CamelFolderSummary *s, gint at_least, CamelException *ex);
+void camel_folder_summary_prepare_fetch_all (CamelFolderSummary *s, CamelException *ex);
 /* insert mi to summary */
 void camel_folder_summary_insert (CamelFolderSummary *s, CamelMessageInfo *info, gboolean load);
 
@@ -514,9 +535,9 @@ void camel_message_info_dump (CamelMessageInfo *mi);
 /* Migration code */
 gint camel_folder_summary_migrate_infos(CamelFolderSummary *s);
 
-void camel_folder_summary_lock_summary (CamelFolderSummary *summary);
-void camel_folder_summary_unlock_summary (CamelFolderSummary *summary);
+void camel_folder_summary_lock   (CamelFolderSummary *summary, CamelFolderSummaryLock lock);
+void camel_folder_summary_unlock (CamelFolderSummary *summary, CamelFolderSummaryLock lock);
 
 G_END_DECLS
 
-#endif /* _CAMEL_FOLDER_SUMMARY_H */
+#endif /* CAMEL_FOLDER_SUMMARY_H */
