@@ -75,7 +75,7 @@ simple_data_wrapper_construct_from_parser (CamelDataWrapper *dw, CamelMimeParser
 
 	mem = camel_stream_mem_new_with_byte_array (buffer);
 	camel_data_wrapper_construct_from_stream (dw, mem);
-	camel_object_unref (mem);
+	g_object_unref (mem);
 }
 
 /**
@@ -90,11 +90,7 @@ camel_mime_part_construct_content_from_parser (CamelMimePart *dw, CamelMimeParse
 	CamelContentType *ct;
 	gchar *encoding;
 
-	/* This replaces the data wrapper repository ... and/or could be
-	 * replaced by it? */
-
-	if (!dw)
-		return;
+	g_return_if_fail (CAMEL_IS_MIME_PART (dw));
 
 	ct = camel_mime_parser_content_type (mp);
 
@@ -139,8 +135,8 @@ camel_mime_part_construct_content_from_parser (CamelMimePart *dw, CamelMimeParse
 
 		/* would you believe you have to set this BEFORE you set the content object???  oh my god !!!! */
 		camel_data_wrapper_set_mime_type_field (content, camel_mime_part_get_content_type (dw));
-		camel_medium_set_content_object ((CamelMedium *)dw, content);
-		camel_object_unref (content);
+		camel_medium_set_content ((CamelMedium *)dw, content);
+		g_object_unref (content);
 	}
 
 	g_free (encoding);
@@ -152,15 +148,16 @@ camel_mime_part_construct_content_from_parser (CamelMimePart *dw, CamelMimeParse
  * Since: 2.28
  **/
 gboolean
-camel_mime_message_build_preview (CamelMimePart *msg, CamelMessageInfo *info)
+camel_mime_message_build_preview (CamelMimePart *msg,
+                                  CamelMessageInfo *info)
 {
 	CamelDataWrapper *dw;
 	gboolean got_plain = FALSE;
 
-	dw = camel_medium_get_content_object((CamelMedium *)msg);
+	dw = camel_medium_get_content ((CamelMedium *)msg);
 	if (camel_content_type_is (dw->mime_type, "multipart", "*")) {
 		gint i, nparts;
-		CamelMultipart *mp = (CamelMultipart *)camel_medium_get_content_object((CamelMedium *)msg);
+		CamelMultipart *mp = (CamelMultipart *)camel_medium_get_content ((CamelMedium *)msg);
 
 		if (!CAMEL_IS_MULTIPART(mp))
 			g_assert (0);
@@ -216,9 +213,9 @@ camel_mime_message_build_preview (CamelMimePart *msg, CamelMessageInfo *info)
 			((CamelMessageInfoBase *) info)->preview = camel_utf8_make_valid(str->str);
 			g_string_free(str, TRUE);
 
-			camel_object_unref (bstream);
+			g_object_unref (bstream);
 		}
-		camel_object_unref (mstream);
+		g_object_unref (mstream);
 		return TRUE;
 	}
 
