@@ -34,13 +34,11 @@
 #include <unistd.h>
 #include <sys/stat.h>
 
-#include <glib.h>
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
 #include "camel-exception.h"
 #include "camel-file-utils.h"
-#include "camel-private.h"
 #include "camel-session.h"
 #include "camel-store.h"
 #include "camel-string-utils.h"
@@ -51,122 +49,146 @@
 
 #define d(x)
 
-#define CS_CLASS(so) ((CamelSessionClass *)((CamelObject *)so)->klass)
+#define CAMEL_SESSION_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), CAMEL_TYPE_SESSION, CamelSessionPrivate))
 
-static CamelService *get_service (CamelSession *session,
-				  const gchar *url_string,
-				  CamelProviderType type,
-				  CamelException *ex);
-static gchar *get_storage_path (CamelSession *session,
-			       CamelService *service,
-			       CamelException *ex);
+struct _CamelSessionPrivate {
+	GMutex *lock;		/* for locking everything basically */
+	GMutex *thread_lock;	/* locking threads */
 
-static gpointer session_thread_msg_new(CamelSession *session, CamelSessionThreadOps *ops, guint size);
-static void session_thread_msg_free(CamelSession *session, CamelSessionThreadMsg *msg);
-static gint session_thread_queue(CamelSession *session, CamelSessionThreadMsg *msg, gint flags);
-static void session_thread_wait(CamelSession *session, gint id);
-static void session_thread_status(CamelSession *session, CamelSessionThreadMsg *msg, const gchar *text, gint pc);
-static void session_forward_to (CamelSession *session, CamelFolder *folder, CamelMimeMessage *message, const gchar *address, CamelException *ex);
+	gint thread_id;
+	GHashTable *thread_active;
+	GThreadPool *thread_pool;
+
+	GHashTable *thread_msg_op;
+	GHashTable *junk_headers;
+
+	guint check_junk        : 1;
+	guint network_available : 1;
+	guint online            : 1;
+};
+
+enum {
+	PROP_0,
+	PROP_CHECK_JUNK,
+	PROP_NETWORK_AVAILABLE,
+	PROP_ONLINE,
+	PROP_STORAGE_PATH
+};
+
+G_DEFINE_TYPE (CamelSession, camel_session, CAMEL_TYPE_OBJECT)
 
 static void
-camel_session_init (CamelSession *session)
+cs_thread_status (CamelOperation *op,
+                  const gchar *what,
+                  gint pc,
+                  gpointer data)
 {
-	session->online = TRUE;
-	session->network_state = TRUE;
-	session->priv = g_malloc0(sizeof(*session->priv));
+	CamelSessionThreadMsg *msg = data;
+	CamelSessionClass *class;
 
-	session->priv->lock = g_mutex_new();
-	session->priv->thread_lock = g_mutex_new();
-	session->priv->thread_id = 1;
-	session->priv->thread_active = g_hash_table_new(NULL, NULL);
-	session->priv->thread_pool = NULL;
-	session->priv->junk_headers = NULL;
+	class = CAMEL_SESSION_GET_CLASS (msg->session);
+	g_return_if_fail (class->thread_status != NULL);
+
+	class->thread_status (msg->session, msg, what, pc);
 }
 
 static void
-camel_session_finalise (CamelObject *o)
+session_set_property (GObject *object,
+                      guint property_id,
+                      const GValue *value,
+                      GParamSpec *pspec)
 {
-	CamelSession *session = (CamelSession *)o;
+	switch (property_id) {
+		case PROP_CHECK_JUNK:
+			camel_session_set_check_junk (
+				CAMEL_SESSION (object),
+				g_value_get_boolean (value));
+			return;
+
+		case PROP_NETWORK_AVAILABLE:
+			camel_session_set_network_available (
+				CAMEL_SESSION (object),
+				g_value_get_boolean (value));
+			return;
+
+		case PROP_ONLINE:
+			camel_session_set_online (
+				CAMEL_SESSION (object),
+				g_value_get_boolean (value));
+			return;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
+static void
+session_get_property (GObject *object,
+                      guint property_id,
+                      GValue *value,
+                      GParamSpec *pspec)
+{
+	switch (property_id) {
+		case PROP_CHECK_JUNK:
+			g_value_set_boolean (
+				value, camel_session_get_check_junk (
+				CAMEL_SESSION (object)));
+			return;
+
+		case PROP_NETWORK_AVAILABLE:
+			g_value_set_boolean (
+				value, camel_session_get_network_available (
+				CAMEL_SESSION (object)));
+			return;
+
+		case PROP_ONLINE:
+			g_value_set_boolean (
+				value, camel_session_get_online (
+				CAMEL_SESSION (object)));
+			return;
+	}
+
+	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+}
+
+static void
+session_finalize (GObject *object)
+{
+	CamelSession *session = CAMEL_SESSION (object);
 	GThreadPool *thread_pool = session->priv->thread_pool;
 
-	g_hash_table_destroy(session->priv->thread_active);
+	g_hash_table_destroy (session->priv->thread_active);
 
 	if (thread_pool != NULL) {
 		/* there should be no unprocessed tasks */
-		g_assert(g_thread_pool_unprocessed (thread_pool) == 0);
-		g_thread_pool_free(thread_pool, FALSE, FALSE);
+		g_assert (g_thread_pool_unprocessed (thread_pool) == 0);
+		g_thread_pool_free (thread_pool, FALSE, FALSE);
 	}
 
-	g_free(session->storage_path);
+	g_free (session->storage_path);
+
+	g_mutex_free (session->priv->lock);
+	g_mutex_free (session->priv->thread_lock);
 
-	g_mutex_free(session->priv->lock);
-	g_mutex_free(session->priv->thread_lock);
 	if (session->priv->junk_headers) {
 		g_hash_table_remove_all (session->priv->junk_headers);
 		g_hash_table_destroy (session->priv->junk_headers);
 	}
-	g_free(session->priv);
-}
 
-static void
-camel_session_class_init (CamelSessionClass *camel_session_class)
-{
-	/* virtual method definition */
-	camel_session_class->get_service = get_service;
-	camel_session_class->get_storage_path = get_storage_path;
-
-	camel_session_class->thread_msg_new = session_thread_msg_new;
-	camel_session_class->thread_msg_free = session_thread_msg_free;
-	camel_session_class->thread_queue = session_thread_queue;
-	camel_session_class->thread_wait = session_thread_wait;
-	camel_session_class->thread_status = session_thread_status;
-
-	camel_session_class->forward_to = session_forward_to;
-
-	camel_object_class_add_event((CamelObjectClass *)camel_session_class, "online", NULL);
-}
-
-CamelType
-camel_session_get_type (void)
-{
-	static CamelType camel_session_type = CAMEL_INVALID_TYPE;
-
-	if (camel_session_type == CAMEL_INVALID_TYPE) {
-		camel_session_type = camel_type_register (
-			camel_object_get_type (), "CamelSession",
-			sizeof (CamelSession),
-			sizeof (CamelSessionClass),
-			(CamelObjectClassInitFunc) camel_session_class_init,
-			NULL,
-			(CamelObjectInitFunc) camel_session_init,
-			(CamelObjectFinalizeFunc) camel_session_finalise);
-	}
-
-	return camel_session_type;
-}
-
-/**
- * camel_session_construct:
- * @session: a #CamelSession object to construct
- * @storage_path: path to a directory the session can use for
- * persistent storage. (This directory must already exist.)
- *
- * Constructs @session.
- **/
-void
-camel_session_construct (CamelSession *session, const gchar *storage_path)
-{
-	session->storage_path = g_strdup (storage_path);
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (camel_session_parent_class)->finalize (object);
 }
 
 static CamelService *
-get_service (CamelSession *session, const gchar *url_string,
-	     CamelProviderType type, CamelException *ex)
+session_get_service (CamelSession *session,
+                     const gchar *url_string,
+                     CamelProviderType type,
+                     CamelException *ex)
 {
 	CamelURL *url;
 	CamelProvider *provider;
 	CamelService *service;
-	CamelException internal_ex;
 
 	url = camel_url_new (url_string, ex);
 	if (!url)
@@ -176,9 +198,10 @@ get_service (CamelSession *session, const gchar *url_string,
 	   the service in the provider's cache */
 	provider = camel_provider_get(url->protocol, ex);
 	if (provider && !provider->object_types[type]) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
-				      _("No provider available for protocol '%s'"),
-				      url->protocol);
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SERVICE_URL_INVALID,
+			_("No provider available for protocol '%s'"),
+			url->protocol);
 		provider = NULL;
 	}
 
@@ -196,12 +219,9 @@ get_service (CamelSession *session, const gchar *url_string,
 	/* Now look up the service in the provider's cache */
 	service = camel_object_bag_reserve(provider->service_cache[type], url);
 	if (service == NULL) {
-		service = (CamelService *)camel_object_new (provider->object_types[type]);
-		camel_exception_init (&internal_ex);
-		camel_service_construct (service, session, provider, url, &internal_ex);
-		if (camel_exception_is_set (&internal_ex)) {
-			camel_exception_xfer (ex, &internal_ex);
-			camel_object_unref (service);
+		service = g_object_new (provider->object_types[type], NULL);
+		if (!camel_service_construct (service, session, provider, url, ex)) {
+			g_object_unref (service);
 			service = NULL;
 			camel_object_bag_abort(provider->service_cache[type], url);
 		} else {
@@ -214,6 +234,225 @@ get_service (CamelSession *session, const gchar *url_string,
 	return service;
 }
 
+static gchar *
+session_get_storage_path (CamelSession *session,
+                          CamelService *service,
+                          CamelException *ex)
+{
+	gchar *path, *p;
+
+	p = camel_service_get_path (service);
+	path = g_strdup_printf ("%s/%s", session->storage_path, p);
+	g_free (p);
+
+#ifdef G_OS_WIN32
+	if (g_access (path, F_OK) == 0)
+#else
+	if (access (path, F_OK) == 0)
+#endif
+		return path;
+
+	if (g_mkdir_with_parents (path, S_IRWXU) == -1) {
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SYSTEM,
+			_("Could not create directory %s:\n%s"),
+			path, g_strerror (errno));
+		g_free (path);
+		return NULL;
+	}
+
+	return path;
+}
+
+static gpointer
+session_thread_msg_new (CamelSession *session,
+                        CamelSessionThreadOps *ops,
+                        guint size)
+{
+	CamelSessionThreadMsg *m;
+
+	m = g_malloc0(size);
+	m->ops = ops;
+	m->session = g_object_ref (session);
+	m->op = camel_operation_new(cs_thread_status, m);
+	camel_exception_init(&m->ex);
+	camel_session_lock (session, CAMEL_SESSION_THREAD_LOCK);
+	m->id = session->priv->thread_id++;
+	g_hash_table_insert(session->priv->thread_active, GINT_TO_POINTER(m->id), m);
+	camel_session_unlock (session, CAMEL_SESSION_THREAD_LOCK);
+
+	return m;
+}
+
+static void
+session_thread_msg_free (CamelSession *session,
+                         CamelSessionThreadMsg *msg)
+{
+	g_return_if_fail (CAMEL_IS_SESSION (session));
+	g_return_if_fail (msg != NULL && msg->ops != NULL);
+
+	d(printf("free message %p session %p\n", msg, session));
+
+	camel_session_lock (session, CAMEL_SESSION_THREAD_LOCK);
+	g_hash_table_remove(session->priv->thread_active, GINT_TO_POINTER(msg->id));
+	camel_session_unlock (session, CAMEL_SESSION_THREAD_LOCK);
+
+	d(printf("free msg, ops->free = %p\n", msg->ops->free));
+
+	if (msg->ops->free)
+		msg->ops->free(session, msg);
+	if (msg->op)
+		camel_operation_unref(msg->op);
+	camel_exception_clear(&msg->ex);
+	g_object_unref (msg->session);
+	g_free(msg);
+}
+
+static void
+session_thread_proxy (CamelSessionThreadMsg *msg,
+                      CamelSession *session)
+{
+	if (msg->ops->receive) {
+		CamelOperation *oldop;
+
+		oldop = camel_operation_register(msg->op);
+		msg->ops->receive(session, msg);
+		camel_operation_register(oldop);
+	}
+
+	camel_session_thread_msg_free(session, msg);
+}
+
+static gint
+session_thread_queue (CamelSession *session,
+                      CamelSessionThreadMsg *msg,
+                      gint flags)
+{
+	GThreadPool *thread_pool;
+	gint id;
+
+	camel_session_lock (session, CAMEL_SESSION_THREAD_LOCK);
+	thread_pool = session->priv->thread_pool;
+	if (thread_pool == NULL) {
+		thread_pool = g_thread_pool_new (
+			(GFunc) session_thread_proxy,
+			session, 1, FALSE, NULL);
+		session->priv->thread_pool = thread_pool;
+	}
+	camel_session_unlock (session, CAMEL_SESSION_THREAD_LOCK);
+
+	id = msg->id;
+	g_thread_pool_push(thread_pool, msg, NULL);
+
+	return id;
+}
+
+static void
+session_thread_wait (CamelSession *session,
+                     gint id)
+{
+	gint wait;
+
+	/* we just busy wait, only other alternative is to setup a reply port? */
+	do {
+		camel_session_lock (session, CAMEL_SESSION_THREAD_LOCK);
+		wait = g_hash_table_lookup(session->priv->thread_active, GINT_TO_POINTER(id)) != NULL;
+		camel_session_unlock (session, CAMEL_SESSION_THREAD_LOCK);
+		if (wait) {
+			g_usleep(20000);
+		}
+	} while (wait);
+}
+
+static void
+session_thread_status (CamelSession *session,
+                       CamelSessionThreadMsg *msg,
+                       const gchar *text,
+                       gint pc)
+{
+}
+
+static void
+camel_session_class_init (CamelSessionClass *class)
+{
+	GObjectClass *object_class;
+
+	g_type_class_add_private (class, sizeof (CamelSessionPrivate));
+
+	object_class = G_OBJECT_CLASS (class);
+	object_class->set_property = session_set_property;
+	object_class->get_property = session_get_property;
+	object_class->finalize = session_finalize;
+
+	class->get_service = session_get_service;
+	class->get_storage_path = session_get_storage_path;
+	class->thread_msg_new = session_thread_msg_new;
+	class->thread_msg_free = session_thread_msg_free;
+	class->thread_queue = session_thread_queue;
+	class->thread_wait = session_thread_wait;
+	class->thread_status = session_thread_status;
+
+	g_object_class_install_property (
+		object_class,
+		PROP_CHECK_JUNK,
+		g_param_spec_boolean (
+			"check-junk",
+			"Check Junk",
+			"Check incoming messages for junk",
+			FALSE,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_NETWORK_AVAILABLE,
+		g_param_spec_boolean (
+			"network-available",
+			"Network Available",
+			"Whether the network is available",
+			TRUE,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT));
+
+	g_object_class_install_property (
+		object_class,
+		PROP_ONLINE,
+		g_param_spec_boolean (
+			"online",
+			"Online",
+			"Whether the shell is online",
+			TRUE,
+			G_PARAM_READWRITE |
+			G_PARAM_CONSTRUCT));
+}
+
+static void
+camel_session_init (CamelSession *session)
+{
+	session->priv = CAMEL_SESSION_GET_PRIVATE (session);
+
+	session->priv->lock = g_mutex_new();
+	session->priv->thread_lock = g_mutex_new();
+	session->priv->thread_id = 1;
+	session->priv->thread_active = g_hash_table_new(NULL, NULL);
+	session->priv->thread_pool = NULL;
+	session->priv->junk_headers = NULL;
+}
+
+/**
+ * camel_session_construct:
+ * @session: a #CamelSession object to construct
+ * @storage_path: path to a directory the session can use for
+ * persistent storage. (This directory must already exist.)
+ *
+ * Constructs @session.
+ **/
+void
+camel_session_construct (CamelSession *session, const gchar *storage_path)
+{
+	session->storage_path = g_strdup (storage_path);
+}
+
 /**
  * camel_session_get_service:
  * @session: a #CamelSession object
@@ -234,17 +473,25 @@ get_service (CamelSession *session, const gchar *url_string,
  * Returns: the requested #CamelService, or %NULL
  **/
 CamelService *
-camel_session_get_service (CamelSession *session, const gchar *url_string,
-			   CamelProviderType type, CamelException *ex)
+camel_session_get_service (CamelSession *session,
+                           const gchar *url_string,
+                           CamelProviderType type,
+                           CamelException *ex)
 {
+	CamelSessionClass *class;
 	CamelService *service;
 
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
 	g_return_val_if_fail (url_string != NULL, NULL);
 
-	CAMEL_SESSION_LOCK (session, lock);
-	service = CS_CLASS (session)->get_service (session, url_string, type, ex);
-	CAMEL_SESSION_UNLOCK (session, lock);
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_val_if_fail (class->get_service != NULL, NULL);
+
+	camel_session_lock (session, CAMEL_SESSION_SESSION_LOCK);
+
+	service = class->get_service (session, url_string, type, ex);
+
+	camel_session_unlock (session, CAMEL_SESSION_SESSION_LOCK);
 
 	return service;
 }
@@ -264,9 +511,9 @@ camel_session_get_service (CamelSession *session, const gchar *url_string,
  **/
 CamelService *
 camel_session_get_service_connected (CamelSession *session,
-				     const gchar *url_string,
-				     CamelProviderType type,
-				     CamelException *ex)
+                                     const gchar *url_string,
+                                     CamelProviderType type,
+                                     CamelException *ex)
 {
 	CamelService *svc;
 
@@ -276,7 +523,7 @@ camel_session_get_service_connected (CamelSession *session,
 
 	if (svc->status != CAMEL_SERVICE_CONNECTED) {
 		if (camel_service_connect (svc, ex) == FALSE) {
-			camel_object_unref (svc);
+			g_object_unref (svc);
 			return NULL;
 		}
 	}
@@ -284,33 +531,6 @@ camel_session_get_service_connected (CamelSession *session,
 	return svc;
 }
 
-static gchar *
-get_storage_path (CamelSession *session, CamelService *service, CamelException *ex)
-{
-	gchar *path, *p;
-
-	p = camel_service_get_path (service);
-	path = g_strdup_printf ("%s/%s", session->storage_path, p);
-	g_free (p);
-
-#ifdef G_OS_WIN32
-	if (g_access (path, F_OK) == 0)
-#else
-	if (access (path, F_OK) == 0)
-#endif
-		return path;
-
-	if (g_mkdir_with_parents (path, S_IRWXU) == -1) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not create directory %s:\n%s"),
-				      path, g_strerror (errno));
-		g_free (path);
-		return NULL;
-	}
-
-	return path;
-}
-
 /**
  * camel_session_get_storage_path:
  * @session: a #CamelSession object
@@ -327,13 +547,19 @@ get_storage_path (CamelSession *session, CamelService *service, CamelException *
  * occurs.
  **/
 gchar *
-camel_session_get_storage_path (CamelSession *session, CamelService *service,
-				CamelException *ex)
+camel_session_get_storage_path (CamelSession *session,
+                                CamelService *service,
+                                CamelException *ex)
 {
+	CamelSessionClass *class;
+
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
 	g_return_val_if_fail (CAMEL_IS_SERVICE (service), NULL);
 
-	return CS_CLASS (session)->get_storage_path (session, service, ex);
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_val_if_fail (class->get_storage_path != NULL, NULL);
+
+	return class->get_storage_path (session, service, ex);
 }
 
 /**
@@ -369,16 +595,25 @@ camel_session_get_storage_path (CamelSession *session, CamelService *service,
  * Returns: the authentication information or %NULL
  **/
 gchar *
-camel_session_get_password (CamelSession *session, CamelService *service,
-			    const gchar *domain, const gchar *prompt, const gchar *item,
-			    guint32 flags,
-			    CamelException *ex)
+camel_session_get_password (CamelSession *session,
+                            CamelService *service,
+                            const gchar *domain,
+                            const gchar *prompt,
+                            const gchar *item,
+                            guint32 flags,
+                            CamelException *ex)
 {
+	CamelSessionClass *class;
+
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
 	g_return_val_if_fail (prompt != NULL, NULL);
 	g_return_val_if_fail (item != NULL, NULL);
 
-	return CS_CLASS (session)->get_password (session, service, domain, prompt, item, flags, ex);
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_val_if_fail (class->get_password != NULL, NULL);
+
+	return class->get_password (
+		session, service, domain, prompt, item, flags, ex);
 }
 
 /**
@@ -396,15 +631,25 @@ camel_session_get_password (CamelSession *session, CamelService *service,
  *
  * @service and @item identify the rejected authentication information,
  * as with #camel_session_get_password.
+ *
+ * Returns: %TRUE on success, %FALSE on failure
  **/
-void
-camel_session_forget_password (CamelSession *session, CamelService *service,
-			       const gchar *domain, const gchar *item, CamelException *ex)
+gboolean
+camel_session_forget_password (CamelSession *session,
+                               CamelService *service,
+                               const gchar *domain,
+                               const gchar *item,
+                               CamelException *ex)
 {
-	g_return_if_fail (CAMEL_IS_SESSION (session));
-	g_return_if_fail (item != NULL);
+	CamelSessionClass *class;
 
-	CS_CLASS (session)->forget_password (session, service, domain, item, ex);
+	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
+	g_return_val_if_fail (item != NULL, FALSE);
+
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_val_if_fail (class->forget_password, FALSE);
+
+	return class->forget_password (session, service, domain, item, ex);
 }
 
 /**
@@ -422,13 +667,20 @@ camel_session_forget_password (CamelSession *session, CamelService *service,
  * Returns: %TRUE if the user accepts, %FALSE if they cancel.
  */
 gboolean
-camel_session_alert_user (CamelSession *session, CamelSessionAlertType type,
-			  const gchar *prompt, gboolean cancel)
+camel_session_alert_user (CamelSession *session,
+                          CamelSessionAlertType type,
+                          const gchar *prompt,
+                          gboolean cancel)
 {
+	CamelSessionClass *class;
+
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
 	g_return_val_if_fail (prompt != NULL, FALSE);
 
-	return CS_CLASS (session)->alert_user (session, type, prompt, cancel);
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_val_if_fail (class->alert_user != NULL, FALSE);
+
+	return class->alert_user (session, type, prompt, cancel);
 }
 
 /**
@@ -437,11 +689,18 @@ camel_session_alert_user (CamelSession *session, CamelSessionAlertType type,
  * Since: 2.22
  **/
 gboolean
-camel_session_lookup_addressbook (CamelSession *session, const gchar *name)
+camel_session_lookup_addressbook (CamelSession *session,
+                                  const gchar *name)
 {
+	CamelSessionClass *class;
+
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
 	g_return_val_if_fail (name != NULL, FALSE);
-	return CS_CLASS (session)->lookup_addressbook (session, name);
+
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_val_if_fail (class->lookup_addressbook != NULL, FALSE);
+
+	return class->lookup_addressbook (session, name);
 }
 
 /**
@@ -491,30 +750,35 @@ camel_session_build_password_prompt (const gchar *type,
 }
 
 /**
- * camel_session_is_online:
- * @session: a #CamelSession object
+ * camel_session_get_online:
+ * @session: a #CamelSession
  *
  * Returns: whether or not @session is online
  **/
 gboolean
-camel_session_is_online (CamelSession *session)
+camel_session_get_online (CamelSession *session)
 {
-	return session->online;
+	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
+
+	return session->priv->online;
 }
 
 /**
  * camel_session_set_online:
- * @session: a #CamelSession object
+ * @session: a #CamelSession
  * @online: whether or not the session should be online
  *
  * Sets the online status of @session to @online.
  **/
 void
-camel_session_set_online (CamelSession *session, gboolean online)
+camel_session_set_online (CamelSession *session,
+                          gboolean online)
 {
-	session->online = online;
+	g_return_if_fail (CAMEL_IS_SESSION (session));
 
-	camel_object_trigger_event(session, "online", GINT_TO_POINTER(online));
+	session->priv->online = online;
+
+	g_object_notify (G_OBJECT (session), "online");
 }
 
 /**
@@ -527,127 +791,18 @@ camel_session_set_online (CamelSession *session, gboolean online)
  **/
 CamelFilterDriver *
 camel_session_get_filter_driver (CamelSession *session,
-				 const gchar *type,
-				 CamelException *ex)
+                                 const gchar *type,
+                                 CamelException *ex)
 {
-	return CS_CLASS (session)->get_filter_driver (session, type, ex);
-}
+	CamelSessionClass *class;
 
-static void
-cs_thread_status(CamelOperation *op, const gchar *what, gint pc, gpointer data)
-{
-	CamelSessionThreadMsg *m = data;
+	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
+	g_return_val_if_fail (type != NULL, NULL);
 
-	CS_CLASS(m->session)->thread_status(m->session, m, what, pc);
-}
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_val_if_fail (class->get_filter_driver != NULL, NULL);
 
-static gpointer
-session_thread_msg_new (CamelSession *session,
-                        CamelSessionThreadOps *ops,
-                        guint size)
-{
-	CamelSessionThreadMsg *m;
-
-	m = g_malloc0(size);
-	m->ops = ops;
-	m->session = session;
-	camel_object_ref(session);
-	m->op = camel_operation_new(cs_thread_status, m);
-	camel_exception_init(&m->ex);
-	CAMEL_SESSION_LOCK(session, thread_lock);
-	m->id = session->priv->thread_id++;
-	g_hash_table_insert(session->priv->thread_active, GINT_TO_POINTER(m->id), m);
-	CAMEL_SESSION_UNLOCK(session, thread_lock);
-
-	return m;
-}
-
-static void
-session_thread_msg_free (CamelSession *session,
-                         CamelSessionThreadMsg *msg)
-{
-	g_return_if_fail (CAMEL_IS_SESSION (session));
-	g_return_if_fail (msg != NULL && msg->ops != NULL);
-
-	d(printf("free message %p session %p\n", msg, session));
-
-	CAMEL_SESSION_LOCK(session, thread_lock);
-	g_hash_table_remove(session->priv->thread_active, GINT_TO_POINTER(msg->id));
-	CAMEL_SESSION_UNLOCK(session, thread_lock);
-
-	d(printf("free msg, ops->free = %p\n", msg->ops->free));
-
-	if (msg->ops->free)
-		msg->ops->free(session, msg);
-	if (msg->op)
-		camel_operation_unref(msg->op);
-	camel_exception_clear(&msg->ex);
-	camel_object_unref(msg->session);
-	g_free(msg);
-}
-
-static void
-session_thread_proxy (CamelSessionThreadMsg *msg,
-                      CamelSession *session)
-{
-	if (msg->ops->receive) {
-		CamelOperation *oldop;
-
-		oldop = camel_operation_register(msg->op);
-		msg->ops->receive(session, msg);
-		camel_operation_register(oldop);
-	}
-
-	camel_session_thread_msg_free(session, msg);
-}
-
-static gint
-session_thread_queue (CamelSession *session,
-                      CamelSessionThreadMsg *msg,
-                      gint flags)
-{
-	GThreadPool *thread_pool;
-	gint id;
-
-	CAMEL_SESSION_LOCK(session, thread_lock);
-	thread_pool = session->priv->thread_pool;
-	if (thread_pool == NULL) {
-		thread_pool = g_thread_pool_new (
-			(GFunc) session_thread_proxy,
-			session, 1, FALSE, NULL);
-		session->priv->thread_pool = thread_pool;
-	}
-	CAMEL_SESSION_UNLOCK(session, thread_lock);
-
-	id = msg->id;
-	g_thread_pool_push(thread_pool, msg, NULL);
-
-	return id;
-}
-
-static void
-session_thread_wait (CamelSession *session,
-                     gint id)
-{
-	gint wait;
-
-	/* we just busy wait, only other alternative is to setup a reply port? */
-	do {
-		CAMEL_SESSION_LOCK(session, thread_lock);
-		wait = g_hash_table_lookup(session->priv->thread_active, GINT_TO_POINTER(id)) != NULL;
-		CAMEL_SESSION_UNLOCK(session, thread_lock);
-		if (wait) {
-			g_usleep(20000);
-		}
-	} while (wait);
-}
-
-static void
-session_thread_status (CamelSession *session,
-                       CamelSessionThreadMsg *msg,
-                       const gchar *text,
-                       gint pc)
-{
+	return class->get_filter_driver (session, type, ex);
 }
 
 /**
@@ -669,11 +824,16 @@ camel_session_thread_msg_new (CamelSession *session,
                               CamelSessionThreadOps *ops,
                               guint size)
 {
+	CamelSessionClass *class;
+
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), NULL);
 	g_return_val_if_fail (ops != NULL, NULL);
 	g_return_val_if_fail (size >= sizeof (CamelSessionThreadMsg), NULL);
 
-	return CS_CLASS (session)->thread_msg_new (session, ops, size);
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_val_if_fail (class->thread_msg_new != NULL, NULL);
+
+	return class->thread_msg_new (session, ops, size);
 }
 
 /**
@@ -688,10 +848,15 @@ void
 camel_session_thread_msg_free (CamelSession *session,
                                CamelSessionThreadMsg *msg)
 {
+	CamelSessionClass *class;
+
 	g_return_if_fail (CAMEL_IS_SESSION (session));
 	g_return_if_fail (msg != NULL && msg->ops != NULL);
 
-	CS_CLASS (session)->thread_msg_free (session, msg);
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_if_fail (class->thread_msg_free != NULL);
+
+	class->thread_msg_free (session, msg);
 }
 
 /**
@@ -711,10 +876,15 @@ camel_session_thread_queue (CamelSession *session,
                             CamelSessionThreadMsg *msg,
                             gint flags)
 {
+	CamelSessionClass *class;
+
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), -1);
 	g_return_val_if_fail (msg != NULL, -1);
 
-	return CS_CLASS (session)->thread_queue (session, msg, flags);
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_val_if_fail (class->thread_queue != NULL, -1);
+
+	return class->thread_queue (session, msg, flags);
 }
 
 /**
@@ -728,34 +898,39 @@ void
 camel_session_thread_wait (CamelSession *session,
                            gint id)
 {
+	CamelSessionClass *class;
+
 	g_return_if_fail (CAMEL_IS_SESSION (session));
 
 	if (id == -1)
 		return;
 
-	CS_CLASS (session)->thread_wait (session, id);
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_if_fail (class->thread_wait != NULL);
+
+	class->thread_wait (session, id);
 }
 
 /**
- * camel_session_check_junk:
- * @session: a #CamelSession object
+ * camel_session_get_check_junk:
+ * @session: a #CamelSession
  *
  * Do we have to check incoming messages to be junk?
  *
  * Returns: whether or not we are checking incoming messages for junk
  **/
 gboolean
-camel_session_check_junk (CamelSession *session)
+camel_session_get_check_junk (CamelSession *session)
 {
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
 
-	return session->check_junk;
+	return session->priv->check_junk;
 }
 
 /**
  * camel_session_set_check_junk:
- * @session: a #CamelSession object
- * @check_junk: state
+ * @session: a #CamelSession
+ * @check_junk: whether to check incoming messages for junk
  *
  * Set check_junk flag, if set, incoming mail will be checked for being junk.
  **/
@@ -765,24 +940,41 @@ camel_session_set_check_junk (CamelSession *session,
 {
 	g_return_if_fail (CAMEL_IS_SESSION (session));
 
-	session->check_junk = check_junk;
+	session->priv->check_junk = check_junk;
+
+	g_object_notify (G_OBJECT (session), "check-junk");
 }
 
+/**
+ * camel_session_get_network_available:
+ * @session: a #CamelSession
+ *
+ * Since: 3.0
+ **/
 gboolean
-camel_session_get_network_state (CamelSession *session)
+camel_session_get_network_available (CamelSession *session)
 {
 	g_return_val_if_fail (CAMEL_IS_SESSION (session), FALSE);
 
-	return session->network_state;
+	return session->priv->network_available;
 }
 
+/**
+ * camel_session_set_network_available:
+ * @session: a #CamelSession
+ * @network_available: whether a network is available
+ *
+ * Since: 3.0
+ **/
 void
-camel_session_set_network_state (CamelSession *session,
-                                 gboolean network_state)
+camel_session_set_network_available (CamelSession *session,
+                                     gboolean network_available)
 {
 	g_return_if_fail (CAMEL_IS_SESSION (session));
 
-	session->network_state = network_state;
+	session->priv->network_available = network_available;
+
+	g_object_notify (G_OBJECT (session), "network-available");
 }
 
 /**
@@ -825,13 +1017,6 @@ camel_session_get_junk_headers (CamelSession *session)
 	return session->priv->junk_headers;
 }
 
-static void
-session_forward_to (CamelSession *session, CamelFolder *folder, CamelMimeMessage *message, const gchar *address, CamelException *ex)
-{
-	if (ex)
-		camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Camel session doesn't support forwarding of a message."));
-}
-
 /**
  * camel_session_forward_to:
  * Forwards message to some address(es) in a given type. The meaning of the forward_type defines session itself.
@@ -844,9 +1029,75 @@ session_forward_to (CamelSession *session, CamelFolder *folder, CamelMimeMessage
  * Since: 2.26
  **/
 void
-camel_session_forward_to (CamelSession *session, CamelFolder *folder, CamelMimeMessage *message, const gchar *address, CamelException *ex)
+camel_session_forward_to (CamelSession *session,
+                          CamelFolder *folder,
+                          CamelMimeMessage *message,
+                          const gchar *address,
+                          CamelException *ex)
+{
+	CamelSessionClass *class;
+
+	g_return_if_fail (CAMEL_IS_SESSION (session));
+	g_return_if_fail (CAMEL_IS_FOLDER (folder));
+	g_return_if_fail (CAMEL_IS_MIME_MESSAGE (message));
+	g_return_if_fail (address != NULL);
+
+	class = CAMEL_SESSION_GET_CLASS (session);
+	g_return_if_fail (class->forward_to != NULL);
+
+	class->forward_to (session, folder, message, address, ex);
+}
+
+/**
+ * camel_session_lock:
+ * @session: a #CamelSession
+ * @lock: lock type to lock
+ *
+ * Locks #session's #lock. Unlock it with camel_session_unlock().
+ *
+ * Since: 3.0
+ **/
+void
+camel_session_lock (CamelSession *session,
+                    CamelSessionLock lock)
 {
 	g_return_if_fail (CAMEL_IS_SESSION (session));
 
-	CS_CLASS (session)->forward_to (session, folder, message, address, ex);
+	switch (lock) {
+		case CAMEL_SESSION_SESSION_LOCK:
+			g_mutex_lock (session->priv->lock);
+			break;
+		case CAMEL_SESSION_THREAD_LOCK:
+			g_mutex_lock (session->priv->thread_lock);
+			break;
+		default:
+			g_return_if_reached ();
+	}
+}
+
+/**
+ * camel_session_unlock:
+ * @session: a #CamelSession
+ * @lock: lock type to unlock
+ *
+ * Unlocks #session's #lock, previously locked with camel_session_lock().
+ *
+ * Since: 3.0
+ **/
+void
+camel_session_unlock (CamelSession *session,
+                      CamelSessionLock lock)
+{
+	g_return_if_fail (CAMEL_IS_SESSION (session));
+
+	switch (lock) {
+		case CAMEL_SESSION_SESSION_LOCK:
+			g_mutex_unlock (session->priv->lock);
+			break;
+		case CAMEL_SESSION_THREAD_LOCK:
+			g_mutex_unlock (session->priv->thread_lock);
+			break;
+		default:
+			g_return_if_reached ();
+	}
 }
