@@ -22,7 +22,6 @@
  * USA
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -31,53 +30,64 @@
 
 #include "camel-stream.h"
 
-static CamelObjectClass *parent_class = NULL;
+G_DEFINE_ABSTRACT_TYPE (CamelStream, camel_stream, CAMEL_TYPE_OBJECT)
 
-/* Returns the class for a CamelStream */
-#define CS_CLASS(so) CAMEL_STREAM_CLASS(CAMEL_OBJECT_GET_CLASS(so))
+static gssize
+stream_read (CamelStream *stream,
+             gchar *buffer,
+             gsize n)
+{
+	return 0;
+}
 
-/* default implementations, do very little */
-static ssize_t   stream_read       (CamelStream *stream, char *buffer, size_t n) { return 0; }
-static ssize_t   stream_write      (CamelStream *stream, const char *buffer, size_t n) { return n; }
-static int       stream_close      (CamelStream *stream) { return 0; }
-static int       stream_flush      (CamelStream *stream) { return 0; }
-static gboolean  stream_eos        (CamelStream *stream) { return stream->eos; }
-static int       stream_reset      (CamelStream *stream) { return 0; }
+static gssize
+stream_write (CamelStream *stream,
+              const gchar *buffer,
+              gsize n)
+{
+	return n;
+}
+
+static gint
+stream_close (CamelStream *stream)
+{
+	return 0;
+}
+
+static gint
+stream_flush (CamelStream *stream)
+{
+	return 0;
+}
+
+static gboolean
+stream_eos (CamelStream *stream)
+{
+	return stream->eos;
+}
+
+static gint
+stream_reset (CamelStream *stream)
+{
+	return 0;
+}
 
 static void
-camel_stream_class_init (CamelStreamClass *camel_stream_class)
+camel_stream_class_init (CamelStreamClass *class)
 {
-	parent_class = camel_type_get_global_classfuncs( CAMEL_OBJECT_TYPE );
-
-	/* virtual method definition */
-	camel_stream_class->read = stream_read;
-	camel_stream_class->write = stream_write;
-	camel_stream_class->close = stream_close;
-	camel_stream_class->flush = stream_flush;
-	camel_stream_class->eos = stream_eos;
-	camel_stream_class->reset = stream_reset;
+	class->read = stream_read;
+	class->write = stream_write;
+	class->close = stream_close;
+	class->flush = stream_flush;
+	class->eos = stream_eos;
+	class->reset = stream_reset;
 }
 
-CamelType
-camel_stream_get_type (void)
+static void
+camel_stream_init (CamelStream *stream)
 {
-	static CamelType camel_stream_type = CAMEL_INVALID_TYPE;
-
-	if (camel_stream_type == CAMEL_INVALID_TYPE) {
-		camel_stream_type = camel_type_register( CAMEL_OBJECT_TYPE,
-							 "CamelStream",
-							 sizeof( CamelStream ),
-							 sizeof( CamelStreamClass ),
-							 (CamelObjectClassInitFunc) camel_stream_class_init,
-							 NULL,
-							 NULL,
-							 NULL );
-	}
-
-	return camel_stream_type;
 }
 
-
 /**
  * camel_stream_read:
  * @stream: a #CamelStream object.
@@ -89,15 +99,21 @@ camel_stream_get_type (void)
  * Returns: the number of bytes actually read, or %-1 on error and set
  * errno.
  **/
-ssize_t
-camel_stream_read (CamelStream *stream, char *buffer, size_t n)
+gssize
+camel_stream_read (CamelStream *stream,
+                   gchar *buffer,
+                   gsize n)
 {
+	CamelStreamClass *class;
+
 	g_return_val_if_fail (CAMEL_IS_STREAM (stream), -1);
 	g_return_val_if_fail (n == 0 || buffer, -1);
-	
-	return (CS_CLASS (stream)->read) (stream, buffer, n);
-}
 
+	class = CAMEL_STREAM_GET_CLASS (stream);
+	g_return_val_if_fail (class->read != NULL, -1);
+
+	return class->read (stream, buffer, n);
+}
 
 /**
  * camel_stream_write:
@@ -110,15 +126,21 @@ camel_stream_read (CamelStream *stream, char *buffer, size_t n)
  * Returns: the number of bytes written to the stream, or %-1 on error
  * along with setting errno.
  **/
-ssize_t
-camel_stream_write (CamelStream *stream, const char *buffer, size_t n)
+gssize
+camel_stream_write (CamelStream *stream,
+                    const gchar *buffer,
+                    gsize n)
 {
+	CamelStreamClass *class;
+
 	g_return_val_if_fail (CAMEL_IS_STREAM (stream), -1);
 	g_return_val_if_fail (n == 0 || buffer, -1);
 
-	return CS_CLASS (stream)->write (stream, buffer, n);
-}
+	class = CAMEL_STREAM_GET_CLASS (stream);
+	g_return_val_if_fail (class->write != NULL, -1);
 
+	return class->write (stream, buffer, n);
+}
 
 /**
  * camel_stream_flush:
@@ -129,31 +151,39 @@ camel_stream_write (CamelStream *stream, const char *buffer, size_t n)
  *
  * Returns: %0 on success or %-1 on fail along with setting errno.
  **/
-int
+gint
 camel_stream_flush (CamelStream *stream)
 {
+	CamelStreamClass *class;
+
 	g_return_val_if_fail (CAMEL_IS_STREAM (stream), -1);
 
-	return CS_CLASS (stream)->flush (stream);
-}
+	class = CAMEL_STREAM_GET_CLASS (stream);
+	g_return_val_if_fail (class->flush != NULL, -1);
 
+	return class->flush (stream);
+}
 
 /**
  * camel_stream_close:
  * @stream: a #CamelStream object
- * 
+ *
  * Closes the stream.
- * 
+ *
  * Returns: %0 on success or %-1 on error.
  **/
-int
+gint
 camel_stream_close (CamelStream *stream)
 {
-	g_return_val_if_fail (CAMEL_IS_STREAM (stream), -1);
-	
-	return CS_CLASS (stream)->close (stream);
-}
+	CamelStreamClass *class;
 
+	g_return_val_if_fail (CAMEL_IS_STREAM (stream), -1);
+
+	class = CAMEL_STREAM_GET_CLASS (stream);
+	g_return_val_if_fail (class->close != NULL, -1);
+
+	return class->close (stream);
+}
 
 /**
  * camel_stream_eos:
@@ -166,11 +196,15 @@ camel_stream_close (CamelStream *stream)
 gboolean
 camel_stream_eos (CamelStream *stream)
 {
-	g_return_val_if_fail (CAMEL_IS_STREAM (stream), TRUE);
-	
-	return CS_CLASS (stream)->eos (stream);
-}
+	CamelStreamClass *class;
 
+	g_return_val_if_fail (CAMEL_IS_STREAM (stream), TRUE);
+
+	class = CAMEL_STREAM_GET_CLASS (stream);
+	g_return_val_if_fail (class->eos != NULL, TRUE);
+
+	return class->eos (stream);
+}
 
 /**
  * camel_stream_reset:
@@ -182,12 +216,17 @@ camel_stream_eos (CamelStream *stream)
  *
  * Returns: %0 on success or %-1 on error along with setting errno.
  **/
-int
+gint
 camel_stream_reset (CamelStream *stream)
 {
+	CamelStreamClass *class;
+
 	g_return_val_if_fail (CAMEL_IS_STREAM (stream), -1);
-	
-	return CS_CLASS (stream)->reset (stream);
+
+	class = CAMEL_STREAM_GET_CLASS (stream);
+	g_return_val_if_fail (class->reset != NULL, -1);
+
+	return class->reset (stream);
 }
 
 /***************** Utility functions ********************/
@@ -201,13 +240,13 @@ camel_stream_reset (CamelStream *stream)
  *
  * Returns: the number of characters written or %-1 on error.
  **/
-ssize_t
-camel_stream_write_string (CamelStream *stream, const char *string)
+gssize
+camel_stream_write_string (CamelStream *stream,
+                           const gchar *string)
 {
 	return camel_stream_write (stream, string, strlen (string));
 }
 
-
 /**
  * camel_stream_printf:
  * @stream: a #CamelStream object
@@ -217,28 +256,30 @@ camel_stream_write_string (CamelStream *stream, const char *string)
  *
  * Returns: the number of characters written or %-1 on error.
  **/
-ssize_t
-camel_stream_printf (CamelStream *stream, const char *fmt, ... )
+gssize
+camel_stream_printf (CamelStream *stream,
+                     const gchar *fmt,
+                     ...)
 {
 	va_list args;
-	char *string;
-	ssize_t ret;
-	
+	gchar *string;
+	gssize ret;
+
 	g_return_val_if_fail (CAMEL_IS_STREAM (stream), -1);
-	
+
 	va_start (args, fmt);
 	string = g_strdup_vprintf (fmt, args);
 	va_end (args);
-	
-	if (!string)
+
+	if (string == NULL)
 		return -1;
-	
+
 	ret = camel_stream_write (stream, string, strlen (string));
 	g_free (string);
+
 	return ret;
 }
 
-
 /**
  * camel_stream_write_to_stream:
  * @stream: source #CamelStream object
@@ -250,27 +291,30 @@ camel_stream_printf (CamelStream *stream, const char *fmt, ... )
  * Returns: %-1 on error, or the number of bytes succesfully
  * copied across streams.
  **/
-ssize_t
-camel_stream_write_to_stream (CamelStream *stream, CamelStream *output_stream)
+gssize
+camel_stream_write_to_stream (CamelStream *stream,
+                              CamelStream *output_stream)
 {
-	char tmp_buf[4096];
-	ssize_t total = 0;
-	ssize_t nb_read;
-	ssize_t nb_written;
+	gchar tmp_buf[4096];
+	gssize total = 0;
+	gssize nb_read;
+	gssize nb_written;
 
 	g_return_val_if_fail (CAMEL_IS_STREAM (stream), -1);
 	g_return_val_if_fail (CAMEL_IS_STREAM (output_stream), -1);
 
 	while (!camel_stream_eos (stream)) {
-		nb_read = camel_stream_read (stream, tmp_buf, sizeof (tmp_buf));
+		nb_read = camel_stream_read (
+			stream, tmp_buf, sizeof (tmp_buf));
 		if (nb_read < 0)
 			return -1;
 		else if (nb_read > 0) {
 			nb_written = 0;
 
 			while (nb_written < nb_read) {
-				ssize_t len = camel_stream_write (output_stream, tmp_buf + nb_written,
-								  nb_read - nb_written);
+				gssize len = camel_stream_write (
+					output_stream, tmp_buf + nb_written,
+					nb_read - nb_written);
 				if (len < 0)
 					return -1;
 				nb_written += len;
