@@ -32,21 +32,17 @@
 #include <unistd.h>
 #include <sys/stat.h>
 
-#include <glib.h>
 #include <glib/gstdio.h>
 
-#include <libedataserver/e-data-server-util.h>
-
 #include "camel-file-utils.h"
-#include "camel-private.h"
 #include "camel-uid-cache.h"
+#include "camel-win32.h"
 
 struct _uid_state {
 	gint level;
 	gboolean save;
 };
 
-
 /**
  * camel_uid_cache_new:
  * @filename: path to load the cache from
@@ -55,7 +51,7 @@ struct _uid_state {
  * doesn't already exist, the UID cache will be empty. Otherwise, if
  * it does exist but can't be read, the function will return %NULL.
  *
- * Return value: a new UID cache, or %NULL
+ * Returns: a new UID cache, or %NULL
  **/
 CamelUIDCache *
 camel_uid_cache_new (const gchar *filename)
@@ -66,7 +62,7 @@ camel_uid_cache_new (const gchar *filename)
 	gint fd, i;
 
 	dirname = g_path_get_dirname (filename);
-	if (g_mkdir_with_parents (dirname, 0777) == -1) {
+	if (g_mkdir_with_parents (dirname, 0700) == -1) {
 		g_free (dirname);
 		return NULL;
 	}
@@ -118,7 +114,6 @@ camel_uid_cache_new (const gchar *filename)
 	return cache;
 }
 
-
 static void
 maybe_write_uid (gpointer key, gpointer value, gpointer data)
 {
@@ -142,14 +137,13 @@ maybe_write_uid (gpointer key, gpointer value, gpointer data)
 	}
 }
 
-
 /**
  * camel_uid_cache_save:
  * @cache: a CamelUIDCache
  *
  * Attempts to save @cache back to disk.
  *
- * Return value: success or failure
+ * Returns: success or failure
  **/
 gboolean
 camel_uid_cache_save (CamelUIDCache *cache)
@@ -233,7 +227,6 @@ camel_uid_cache_save (CamelUIDCache *cache)
 	return FALSE;
 }
 
-
 static void
 free_uid (gpointer key, gpointer value, gpointer data)
 {
@@ -241,7 +234,6 @@ free_uid (gpointer key, gpointer value, gpointer data)
 	g_free (value);
 }
 
-
 /**
  * camel_uid_cache_destroy:
  * @cache: a CamelUIDCache
@@ -257,7 +249,6 @@ camel_uid_cache_destroy (CamelUIDCache *cache)
 	g_free (cache);
 }
 
-
 /**
  * camel_uid_cache_get_new_uids:
  * @cache: a CamelUIDCache
@@ -266,7 +257,7 @@ camel_uid_cache_destroy (CamelUIDCache *cache)
  * Returns an array of UIDs from @uids that are not in @cache, and
  * removes UIDs from @cache that aren't in @uids.
  *
- * Return value: an array of new UIDs, which must be freed with
+ * Returns: an array of new UIDs, which must be freed with
  * camel_uid_cache_free_uids().
  **/
 GPtrArray *
@@ -300,7 +291,6 @@ camel_uid_cache_get_new_uids (CamelUIDCache *cache, GPtrArray *uids)
 	return new_uids;
 }
 
-
 /**
  * camel_uid_cache_save_uid:
  * @cache: a CamelUIDCache
@@ -328,7 +318,6 @@ camel_uid_cache_save_uid (CamelUIDCache *cache, const gchar *uid)
 	}
 }
 
-
 /**
  * camel_uid_cache_free_uids:
  * @uids: an array returned from camel_uid_cache_get_new_uids()
