@@ -25,21 +25,17 @@
 #include <config.h>
 
 #include <string.h>
-
+#include <camel/camel.h>
 #include <glib/gi18n-lib.h>
 
 #include "camel-imap-store.h"
-#include "camel-provider.h"
-#include "camel-sasl.h"
-#include "camel-session.h"
-#include "camel-url.h"
 
-static void add_hash (guint *hash, char *s);
+static void add_hash (guint *hash, gchar *s);
 static guint imap_url_hash (gconstpointer key);
-static gint check_equal (char *s1, char *s2);
+static gint check_equal (gchar *s1, gchar *s2);
 static gint imap_url_equal (gconstpointer a, gconstpointer b);
 
-CamelProviderConfEntry imap_conf_entries[] = {
+static CamelProviderConfEntry imap_conf_entries[] = {
 	{ CAMEL_PROVIDER_CONF_SECTION_START, "mailcheck", NULL,
 	  N_("Checking for New Mail") },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "check_all", NULL,
@@ -53,7 +49,7 @@ CamelProviderConfEntry imap_conf_entries[] = {
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "use_command", NULL,
 	  N_("_Use custom command to connect to server"), "0" },
 	{ CAMEL_PROVIDER_CONF_ENTRY, "command", "use_command",
-	  N_("Command:"), "ssh -C -l %u %h exec /usr/sbin/imapd" },
+	  N_("Co_mmand:"), "ssh -C -l %u %h exec /usr/sbin/imapd" },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 #endif
 	{ CAMEL_PROVIDER_CONF_SECTION_START, "folders", NULL,
@@ -63,7 +59,7 @@ CamelProviderConfEntry imap_conf_entries[] = {
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "override_namespace", NULL,
 	  N_("O_verride server-supplied folder namespace"), "0" },
 	{ CAMEL_PROVIDER_CONF_ENTRY, "namespace", "override_namespace",
-	  N_("Namespace:") },
+	  N_("Names_pace:") },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 	{ CAMEL_PROVIDER_CONF_SECTION_START, "general", NULL, N_("Options") },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "filter", NULL,
@@ -72,7 +68,7 @@ CamelProviderConfEntry imap_conf_entries[] = {
 	  N_("Check new messages for Jun_k contents"), "0" },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "filter_junk_inbox", "filter_junk",
 	  N_("Only check for Junk messages in the IN_BOX folder"), "0" },
-	{ CAMEL_PROVIDER_CONF_CHECKBOX, "offline_sync", NULL,
+	{ CAMEL_PROVIDER_CONF_CHECKBOX, "sync_offline", NULL,
 	  N_("Automatically synchroni_ze remote mail locally"), "0" },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 	{ CAMEL_PROVIDER_CONF_END }
@@ -98,10 +94,10 @@ static CamelProvider imap_provider = {
 
 CamelServiceAuthType camel_imap_password_authtype = {
 	N_("Password"),
-	
+
 	N_("This option will connect to the IMAP server using a "
 	   "plaintext password."),
-	
+
 	"",
 	TRUE
 };
@@ -120,7 +116,7 @@ camel_provider_module_init(void)
 }
 
 static void
-add_hash (guint *hash, char *s)
+add_hash (guint *hash, gchar *s)
 {
 	if (s)
 		*hash ^= g_str_hash(s);
@@ -135,12 +131,12 @@ imap_url_hash (gconstpointer key)
 	add_hash (&hash, u->user);
 	add_hash (&hash, u->host);
 	hash ^= u->port;
-	
+
 	return hash;
 }
 
-static int
-check_equal (char *s1, char *s2)
+static gint
+check_equal (gchar *s1, gchar *s2)
 {
 	if (s1 == NULL) {
 		if (s2 == NULL)
@@ -148,18 +144,18 @@ check_equal (char *s1, char *s2)
 		else
 			return FALSE;
 	}
-	
+
 	if (s2 == NULL)
 		return FALSE;
-	
+
 	return strcmp (s1, s2) == 0;
 }
 
-static int
+static gint
 imap_url_equal (gconstpointer a, gconstpointer b)
 {
 	const CamelURL *u1 = a, *u2 = b;
-	
+
 	return check_equal (u1->protocol, u2->protocol)
 		&& check_equal (u1->user, u2->user)
 		&& check_equal (u1->host, u2->host)
