@@ -28,33 +28,18 @@
 #include <errno.h>
 #include <glib/gi18n-lib.h>
 
-#include "camel/camel-exception.h"
-#include "camel/camel-stream-mem.h"
-#include "camel/camel-stream-filter.h"
-#include "camel/camel-mime-message.h"
-#include "camel/camel-operation.h"
-#include "camel/camel-data-cache.h"
-#include "camel/camel-session.h"
-#include "camel/camel-file-utils.h"
-#include  "camel/camel-string-utils.h"
-#include "camel-folder-search.h"
-
 #include "camel-imapx-store.h"
 #include "camel-imapx-folder.h"
 #include "camel-imapx-summary.h"
 #include "camel-imapx-exception.h"
 #include "camel-imapx-server.h"
 
-#include <libedataserver/md5-utils.h>
-
 #include <stdlib.h>
 #include <string.h>
 
 #define d(x)
 
-#define CF_CLASS(o) (CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(o)))
-static CamelObjectClass *parent_class;
-static CamelOfflineFolderClass *offline_folder_class = NULL;
+G_DEFINE_TYPE (CamelIMAPXFolder, camel_imapx_folder, CAMEL_TYPE_OFFLINE_FOLDER)
 
 CamelFolder *
 camel_imapx_folder_new(CamelStore *store, const gchar *folder_dir, const gchar *folder_name, CamelException *ex)
@@ -73,8 +58,11 @@ camel_imapx_folder_new(CamelStore *store, const gchar *folder_dir, const gchar *
 	else
 		short_name = folder_name;
 
-	folder = CAMEL_FOLDER (camel_object_new (CAMEL_IMAPX_FOLDER_TYPE));
-	camel_folder_construct(folder, store, folder_name, short_name);
+	folder = g_object_new (
+		CAMEL_TYPE_IMAPX_FOLDER,
+		"name", short_name,
+		"full_name", folder_name,
+		"parent-store", store, NULL);
 	ifolder = (CamelIMAPXFolder *) folder;
 
 	((CamelIMAPXFolder *)folder)->raw_name = g_strdup(folder_name);
@@ -88,7 +76,7 @@ camel_imapx_folder_new(CamelStore *store, const gchar *folder_dir, const gchar *
 		return NULL;
 	}
 
-	ifolder->cache = camel_data_cache_new (folder_dir, 0, ex);
+	ifolder->cache = camel_data_cache_new (folder_dir, ex);
 	if (!ifolder->cache) {
 		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
 				      _("Could not create cache for %s"),
@@ -97,14 +85,14 @@ camel_imapx_folder_new(CamelStore *store, const gchar *folder_dir, const gchar *
 	}
 
 	state_file = g_strdup_printf ("%s/cmeta", folder_dir);
-	camel_object_set(folder, NULL, CAMEL_OBJECT_STATE_FILE, state_file, NULL);
+	camel_object_set_state_filename (CAMEL_OBJECT (folder), state_file);
 	g_free(state_file);
-	camel_object_state_read(folder);
+	camel_object_state_read (CAMEL_OBJECT (folder));
 
 	ifolder->search = camel_folder_search_new ();
 	ifolder->search_lock = g_mutex_new ();
 	ifolder->stream_lock = g_mutex_new ();
-	ifolder->ignore_recent = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free, NULL); 
+	ifolder->ignore_recent = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free, NULL);
 	ifolder->exists_on_server = 0;
 	ifolder->unread_on_server = 0;
 
@@ -115,7 +103,7 @@ camel_imapx_folder_new(CamelStore *store, const gchar *folder_dir, const gchar *
 		if ((istore->rec_options & IMAPX_FILTER_INBOX))
 			folder->folder_flags |= CAMEL_FOLDER_FILTER_JUNK;
 	} else if ((istore->rec_options & (IMAPX_FILTER_JUNK | IMAPX_FILTER_JUNK_INBOX)) == IMAPX_FILTER_JUNK)
-			folder->folder_flags |= CAMEL_FOLDER_FILTER_JUNK;	
+			folder->folder_flags |= CAMEL_FOLDER_FILTER_JUNK;
 
 	g_free (summary_file);
 
@@ -123,39 +111,88 @@ camel_imapx_folder_new(CamelStore *store, const gchar *folder_dir, const gchar *
 }
 
 static void
+imapx_folder_dispose (GObject *object)
+{
+	CamelIMAPXFolder *folder = CAMEL_IMAPX_FOLDER (object);
+
+	if (folder->cache != NULL) {
+		g_object_unref (folder->cache);
+		folder->cache = NULL;
+	}
+
+	if (folder->search != NULL) {
+		g_object_unref (folder->search);
+		folder->search = NULL;
+	}
+
+	/* Chain up to parent's dispose() method. */
+	G_OBJECT_CLASS (camel_imapx_folder_parent_class)->dispose (object);
+}
+
+static void
+imapx_folder_finalize (GObject *object)
+{
+	CamelIMAPXFolder *folder = CAMEL_IMAPX_FOLDER (object);
+
+	if (folder->ignore_recent != NULL)
+		g_hash_table_unref (folder->ignore_recent);
+
+	g_mutex_free (folder->search_lock);
+	g_mutex_free (folder->stream_lock);
+
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (camel_imapx_folder_parent_class)->finalize (object);
+}
+
+static gboolean
 imapx_refresh_info (CamelFolder *folder, CamelException *ex)
 {
-	CamelIMAPXStore *istore = (CamelIMAPXStore *)folder->parent_store;
+	CamelStore *parent_store;
+	CamelIMAPXStore *istore;
+
+	parent_store = camel_folder_get_parent_store (folder);
+	istore = CAMEL_IMAPX_STORE (parent_store);
 
 	if (CAMEL_OFFLINE_STORE (istore)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
-		return;
+		return TRUE;
 
 	camel_service_connect((CamelService *)istore, ex);
 	if (istore->server && camel_imapx_server_connect (istore->server, TRUE, ex))
 		camel_imapx_server_refresh_info(istore->server, folder, ex);
+
+	return !camel_exception_is_set (ex);
 }
 
-static void
+static gboolean
 imapx_expunge (CamelFolder *folder, CamelException *ex)
 {
-	CamelIMAPXStore *is = (CamelIMAPXStore *)folder->parent_store;
+	CamelStore *parent_store;
+	CamelIMAPXStore *is;
+
+	parent_store = camel_folder_get_parent_store (folder);
+	is = CAMEL_IMAPX_STORE (parent_store);
 
 	if (CAMEL_OFFLINE_STORE (is)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
-		return;
+		return TRUE;
 
 	if (is->server && camel_imapx_server_connect (is->server, TRUE, ex))
 		camel_imapx_server_expunge(is->server, folder, ex);
 
+	return !camel_exception_is_set (ex);
 }
 
-static void
+static gboolean
 imapx_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 {
-	CamelIMAPXStore *is = (CamelIMAPXStore *)folder->parent_store;
+	CamelStore *parent_store;
+	CamelIMAPXStore *is;
 	CamelException eex = CAMEL_EXCEPTION_INITIALISER;
 
+	parent_store = camel_folder_get_parent_store (folder);
+	is = CAMEL_IMAPX_STORE (parent_store);
+
 	if (CAMEL_OFFLINE_STORE (is)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
-		return;
+		return TRUE;
 
 	if (!ex)
 		ex = &eex;
@@ -171,6 +208,8 @@ imapx_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 		camel_imapx_server_expunge(is->server, folder, ex);
 		camel_exception_clear(ex);
 	}
+
+	return !camel_exception_is_set (ex);
 }
 
 static CamelMimeMessage *
@@ -178,11 +217,15 @@ imapx_get_message (CamelFolder *folder, const gchar *uid, CamelException *ex)
 {
 	CamelMimeMessage *msg = NULL;
 	CamelStream *stream = NULL;
-	CamelIMAPXStore *istore = (CamelIMAPXStore *)folder->parent_store;
+	CamelStore *parent_store;
+	CamelIMAPXStore *istore;
 	CamelIMAPXFolder *ifolder = (CamelIMAPXFolder *) folder;
 	const gchar *path = NULL;
 	gboolean offline_message = FALSE;
 
+	parent_store = camel_folder_get_parent_store (folder);
+	istore = CAMEL_IMAPX_STORE (parent_store);
+
 	if (!strchr (uid, '-'))
 		path = "cur";
 	else {
@@ -210,60 +253,78 @@ imapx_get_message (CamelFolder *folder, const gchar *uid, CamelException *ex)
 
 	if (!camel_exception_is_set (ex) && stream) {
 		msg = camel_mime_message_new();
-	
+
 		g_mutex_lock (ifolder->stream_lock);
 		if (camel_data_wrapper_construct_from_stream((CamelDataWrapper *)msg, stream) == -1) {
-			camel_object_unref(msg);
+			g_object_unref (msg);
 			msg = NULL;
 		}
 		g_mutex_unlock (ifolder->stream_lock);
-		camel_object_unref(stream);
+		g_object_unref (stream);
 	}
 
 	return msg;
 }
 
-static void
+static gboolean
 imapx_sync_message (CamelFolder *folder, const gchar *uid, CamelException *ex)
 {
-	CamelIMAPXStore *istore = (CamelIMAPXStore *)folder->parent_store;
+	CamelStore *parent_store;
+	CamelIMAPXStore *istore;
+
+	parent_store = camel_folder_get_parent_store (folder);
+	istore = CAMEL_IMAPX_STORE (parent_store);
 
 	if (CAMEL_OFFLINE_STORE (istore)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
-		return;
+		return TRUE;
 
 	if (istore->server && camel_imapx_server_connect (istore->server, TRUE, ex))
 		camel_imapx_server_sync_message (istore->server, folder, uid, ex);
+
+	return !camel_exception_is_set (ex);
 }
 
-static void
+static gboolean
 imapx_transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 		      CamelFolder *dest, GPtrArray **transferred_uids,
 		      gboolean delete_originals, CamelException *ex)
 {
-	CamelIMAPXStore *istore = (CamelIMAPXStore *) source->parent_store;
+	CamelStore *parent_store;
+	CamelIMAPXStore *istore;
+
+	parent_store = camel_folder_get_parent_store (source);
+	istore = CAMEL_IMAPX_STORE (parent_store);
 
 	if (CAMEL_OFFLINE_STORE (istore)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
-		return;
+		return TRUE;
 
 	if (istore->server && camel_imapx_server_connect (istore->server, TRUE, ex))
 		camel_imapx_server_copy_message (istore->server, source, dest, uids, delete_originals, ex);
 
 	imapx_refresh_info (dest, ex);
+
+	return !camel_exception_is_set (ex);
 }
 
-static void
+static gboolean
 imapx_append_message(CamelFolder *folder, CamelMimeMessage *message, const CamelMessageInfo *info, gchar **appended_uid, CamelException *ex)
 {
-	CamelIMAPXStore *istore = (CamelIMAPXStore *)folder->parent_store;
+	CamelStore *parent_store;
+	CamelIMAPXStore *istore;
+
+	parent_store = camel_folder_get_parent_store (folder);
+	istore = CAMEL_IMAPX_STORE (parent_store);
 
 	if (CAMEL_OFFLINE_STORE (istore)->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL)
-		return;
+		return TRUE;
 
 	if (appended_uid)
 		*appended_uid = NULL;
 
 	if (istore->server && camel_imapx_server_connect (istore->server, TRUE, ex))
 		camel_imapx_server_append_message(istore->server, folder, message, info, ex);
+
+	return !camel_exception_is_set (ex);
 }
 
 gchar *
@@ -396,29 +457,34 @@ imapx_search_by_expression (CamelFolder *folder, const gchar *expression, CamelE
 }
 
 static void
-imapx_folder_class_init (CamelIMAPXFolderClass *klass)
+camel_imapx_folder_class_init (CamelIMAPXFolderClass *class)
 {
-	offline_folder_class = CAMEL_OFFLINE_FOLDER_CLASS (camel_type_get_global_classfuncs (camel_offline_folder_get_type ()));
+	GObjectClass *object_class;
+	CamelFolderClass *folder_class;
 
-	((CamelFolderClass *)klass)->refresh_info = imapx_refresh_info;
-	((CamelFolderClass *)klass)->sync = imapx_sync;
-	((CamelFolderClass *)klass)->search_by_expression = imapx_search_by_expression;
-	((CamelFolderClass *)klass)->search_by_uids = imapx_search_by_uids;
-	((CamelFolderClass *)klass)->count_by_expression = imapx_count_by_expression;
-	((CamelFolderClass *)klass)->search_free = imapx_search_free;
+	object_class = G_OBJECT_CLASS (class);
+	object_class->dispose = imapx_folder_dispose;
+	object_class->finalize = imapx_folder_finalize;
 
-	((CamelFolderClass *)klass)->expunge = imapx_expunge;
-	((CamelFolderClass *)klass)->get_message = imapx_get_message;
-	((CamelFolderClass *)klass)->sync_message = imapx_sync_message;
-	((CamelFolderClass *)klass)->append_message = imapx_append_message;
-	((CamelFolderClass *)klass)->transfer_messages_to = imapx_transfer_messages_to;
-	((CamelFolderClass *)klass)->get_filename = imapx_get_filename;
+	folder_class = CAMEL_FOLDER_CLASS (class);
+	folder_class->refresh_info = imapx_refresh_info;
+	folder_class->sync = imapx_sync;
+	folder_class->search_by_expression = imapx_search_by_expression;
+	folder_class->search_by_uids = imapx_search_by_uids;
+	folder_class->count_by_expression = imapx_count_by_expression;
+	folder_class->search_free = imapx_search_free;
+	folder_class->expunge = imapx_expunge;
+	folder_class->get_message = imapx_get_message;
+	folder_class->sync_message = imapx_sync_message;
+	folder_class->append_message = imapx_append_message;
+	folder_class->transfer_messages_to = imapx_transfer_messages_to;
+	folder_class->get_filename = imapx_get_filename;
 }
 
 static void
-imapx_folder_init(CamelObject *o, CamelObjectClass *klass)
+camel_imapx_folder_init (CamelIMAPXFolder *imapx_folder)
 {
-	CamelFolder *folder = (CamelFolder *)o;
+	CamelFolder *folder = CAMEL_FOLDER (imapx_folder);
 
 	folder->folder_flags |= (CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY |
 				 CAMEL_FOLDER_HAS_SEARCH_CAPABILITY);
@@ -430,37 +496,3 @@ imapx_folder_init(CamelObject *o, CamelObjectClass *klass)
 	camel_folder_set_lock_async (folder, TRUE);
 }
 
-static void
-imapx_finalize (CamelObject *object)
-{
-	CamelIMAPXFolder *ifolder = (CamelIMAPXFolder *) object;
-
-	camel_object_unref (CAMEL_OBJECT (ifolder->cache));
-	
-	if (ifolder->ignore_recent)
-		g_hash_table_unref (ifolder->ignore_recent);
-
-	g_mutex_free (ifolder->search_lock);
-	g_mutex_free (ifolder->stream_lock);
-	if (ifolder->search)
-		camel_object_unref (CAMEL_OBJECT (ifolder->search));
-}
-
-CamelType
-camel_imapx_folder_get_type (void)
-{
-	static CamelType camel_imapx_folder_type = CAMEL_INVALID_TYPE;
-
-	if (!camel_imapx_folder_type) {
-		parent_class = camel_offline_folder_get_type();
-		camel_imapx_folder_type = camel_type_register (parent_class, "CamelIMAPXFolder",
-							      sizeof (CamelIMAPXFolder),
-							      sizeof (CamelIMAPXFolderClass),
-							      (CamelObjectClassInitFunc)imapx_folder_class_init,
-							      NULL,
-							      imapx_folder_init,
-							      (CamelObjectFinalizeFunc) imapx_finalize);
-	}
-
-	return camel_imapx_folder_type;
-}
