@@ -3,47 +3,17 @@
 #include <errno.h>
 #include <string.h>
 
-#include <camel/camel-folder-summary.h>
-#include <camel/camel-store.h>
-#include <camel/camel-utf8.h>
-#include <camel/camel-string-utils.h>
-#include <camel/camel-debug.h>
-
 #include "camel-imapx-folder.h"
 #include "camel-imapx-stream.h"
 #include "camel-imapx-summary.h"
 #include "camel-imapx-store-summary.h"
 #include "camel-imapx-utils.h"
 #include "camel-imapx-exception.h"
-#include "libedataserver/e-memory.h"
 
 /* high-level parser state */
-#define p(x) camel_imapx_debug(parse, x)
+#define p(x)
 /* debug */
-#define d(x) camel_imapx_debug(debug, x)
-
-gint camel_imapx_debug_flags;
-
-#define debug_set_flag(flag) do { \
-	if ((CAMEL_IMAPX_DEBUG_ALL & CAMEL_IMAPX_DEBUG_ ## flag) &&	\
-	    camel_debug("imapx:" #flag))				\
-		camel_imapx_debug_flags |= CAMEL_IMAPX_DEBUG_ ## flag;	\
-	} while (0)
-
-static void camel_imapx_set_debug_flags(void)
-{
-	if (camel_debug("imapx")) {
-		camel_imapx_debug_flags = CAMEL_IMAPX_DEBUG_ALL;
-		return;
-	}
-
-	debug_set_flag(command);
-	debug_set_flag(debug);
-	debug_set_flag(extra);
-	debug_set_flag(io);
-	debug_set_flag(token);
-	debug_set_flag(parse);
-}
+#define d(x)
 
 #include "camel-imapx-tokenise.h"
 #define SUBFOLDER_DIR_NAME     "subfolders"
@@ -63,6 +33,7 @@ imapx_tokenise (register const gchar *str, register guint len)
 
 static void imapx_namespace_clear (CamelIMAPXStoreNamespace **ns);
 static const gchar * rename_label_flag (const gchar *flag, gint len, gboolean server_to_evo);
+static GPtrArray *imapx_parse_uids (CamelIMAPXStream *is, CamelException *ex);
 
 /* flag table */
 static struct {
@@ -75,8 +46,6 @@ static struct {
 	{ "\\FLAGGED", CAMEL_MESSAGE_FLAGGED },
 	{ "\\SEEN", CAMEL_MESSAGE_SEEN },
 	{ "\\RECENT", CAMEL_IMAPX_MESSAGE_RECENT },
-	{ "JUNK", CAMEL_MESSAGE_JUNK },
-	{ "NOTJUNK", CAMEL_MESSAGE_NOTJUNK },
 	{ "\\*", CAMEL_MESSAGE_USER }
 };
 
@@ -98,7 +67,7 @@ imapx_parse_flags(CamelIMAPXStream *stream, guint32 *flagsp, CamelFlag **user_fl
 	if (tok == '(') {
 		do {
 			tok = camel_imapx_stream_token(stream, &token, &len, ex);
-			if (tok == IMAPX_TOK_TOKEN || tok == IMAPX_TOK_INT) {
+			if (tok == IMAPX_TOK_TOKEN) {
 				p = token;
 				// FIXME: ascii_toupper
 				while ((c=*p))
@@ -169,7 +138,6 @@ imapx_write_flags(CamelStream *stream, guint32 flags, CamelFlag *user_flags, Cam
 /* throws IO exception */
 {
 	gint i;
-	gboolean first = TRUE;
 
 	if (camel_stream_write(stream, "(", 1) == -1) {
 		camel_exception_setv (ex, 1, "io error: %s", strerror(errno));
@@ -180,29 +148,30 @@ imapx_write_flags(CamelStream *stream, guint32 flags, CamelFlag *user_flags, Cam
 		if (flag_table[i].flag & flags) {
 			if (flags & CAMEL_IMAPX_MESSAGE_RECENT)
 				continue;
-			if (!first && camel_stream_write(stream, " ", 1) == -1) {
-				camel_exception_setv (ex, 1, "io error: %s", strerror(errno));
-				return;
-			}
-			first = FALSE;
+
 			if (camel_stream_write (stream, flag_table[i].name, strlen(flag_table[i].name)) == -1) {
 				camel_exception_setv (ex,1, "io error: %s", strerror(errno));
 				return;
 			}
 
 			flags &= ~flag_table[i].flag;
+			if (flags != 0 && user_flags == NULL)
+				if (camel_stream_write(stream, " ", 1) == -1) {
+					camel_exception_setv (ex, 1, "io error: %s", strerror(errno));
+					return;
+				}
 		}
 	}
 
 	while (user_flags) {
 		const gchar *flag_name = rename_label_flag (user_flags->name, strlen (user_flags->name), FALSE);
 
-		if (!first && camel_stream_write(stream, " ", 1) == -1) {
+		if (camel_stream_write(stream, flag_name, strlen (flag_name)) == -1) {
 			camel_exception_setv (ex, 1, "io error: %s", strerror(errno));
 			return;
 		}
-		first = FALSE;
-		if (camel_stream_write(stream, flag_name, strlen (flag_name)) == -1) {
+
+		if (user_flags->next && camel_stream_write(stream, " ", 1) == -1) {
 			camel_exception_setv (ex, 1, "io error: %s", strerror(errno));
 			return;
 		}
@@ -236,10 +205,9 @@ imapx_update_user_flags (CamelMessageInfo *info, CamelFlag *server_user_flags)
 }
 
 gboolean
-imapx_update_message_info_flags (CamelMessageInfo *info, guint32 server_flags, CamelFlag *server_user_flags, CamelFolder *folder, gboolean unsolicited)
+imapx_update_message_info_flags (CamelMessageInfo *info, guint32 server_flags, CamelFlag *server_user_flags, CamelFolder *folder)
 {
 	gboolean changed = FALSE;
-	CamelIMAPXFolder *ifolder = (CamelIMAPXFolder *)folder;
 	CamelIMAPXMessageInfo *xinfo = (CamelIMAPXMessageInfo *) info;
 
 	if (server_flags != xinfo->server_flags)
@@ -269,11 +237,8 @@ imapx_update_message_info_flags (CamelMessageInfo *info, guint32 server_flags, C
 					deleted == 1 ? "deleted" : ( deleted == -1 ? "undeleted" : ""),
 					junk == 1 ? "junk" : ( junk == -1 ? "unjunked" : "")));
 
-		if (read) {
+		if (read)
 			folder->summary->unread_count -= read;
-			if (unsolicited)
-				ifolder->unread_on_server -= read;
-		}
 		if (deleted)
 			folder->summary->deleted_count += deleted;
 		if (junk)
@@ -345,10 +310,9 @@ imapx_set_message_info_flags_for_new_message (CamelMessageInfo *info, guint32 se
 }
 
 void
-imapx_update_summary_for_removed_message (CamelMessageInfo *info, CamelFolder *folder, gboolean unsolicited)
+imapx_update_summary_for_removed_message (CamelMessageInfo *info, CamelFolder *folder)
 {
 	CamelMessageInfoBase *dinfo = (CamelMessageInfoBase *) info;
-	CamelIMAPXFolder *ifolder = (CamelIMAPXFolder *)folder;
 	gint unread=0, deleted=0, junk=0;
 	guint32 flags;
 
@@ -362,11 +326,9 @@ imapx_update_summary_for_removed_message (CamelMessageInfo *info, CamelFolder *f
 	if (flags & CAMEL_MESSAGE_JUNK)
 		junk = 1;
 
-	if (unread) {
+	if (unread)
 		folder->summary->unread_count--;
-		if (unsolicited)
-			ifolder->unread_on_server--;
-	}
+
 	if (deleted)
 		folder->summary->deleted_count--;
 	if (junk)
@@ -385,18 +347,25 @@ void
 imapx_update_store_summary (CamelFolder *folder)
 {
 	CamelStoreInfo *si;
+	CamelStore *parent_store;
+	const gchar *full_name;
 
-	si = camel_store_summary_path ((CamelStoreSummary *) ((CamelIMAPXStore *) folder->parent_store)->summary, folder->full_name);
+	full_name = camel_folder_get_full_name (folder);
+	parent_store = camel_folder_get_parent_store (folder);
+
+	si = camel_store_summary_path ((CamelStoreSummary *) ((CamelIMAPXStore *) parent_store)->summary, full_name);
 	if (si) {
 		guint32 unread, total;
 
-		camel_object_get(folder, NULL, CAMEL_FOLDER_TOTAL, &total, CAMEL_FOLDER_UNREAD, &unread, NULL);
+		total = camel_folder_summary_count (folder->summary);
+		unread = folder->summary->unread_count;
+
 		if (si->unread != unread || si->total != total) {
 			si->unread = unread;
 			si->total = total;
 
-			camel_store_summary_touch ((CamelStoreSummary *)((CamelIMAPXStore *) folder->parent_store)->summary);
-			camel_store_summary_save ((CamelStoreSummary *)((CamelIMAPXStore *) folder->parent_store)->summary);
+			camel_store_summary_touch ((CamelStoreSummary *)((CamelIMAPXStore *) parent_store)->summary);
+			camel_store_summary_save ((CamelStoreSummary *)((CamelIMAPXStore *) parent_store)->summary);
 		}
 	}
 }
@@ -421,10 +390,6 @@ struct {
 	{ "LITERAL+", IMAPX_CAPABILITY_LITERALPLUS },
 	{ "STARTTLS", IMAPX_CAPABILITY_STARTTLS },
 	{ "IDLE", IMAPX_CAPABILITY_IDLE },
-	{ "CONDSTORE", IMAPX_CAPABILITY_CONDSTORE },
-	{ "QRESYNC", IMAPX_CAPABILITY_QRESYNC },
-	{ "LIST-EXTENDED", IMAPX_CAPABILITY_LIST_EXTENDED },
-	{ "LIST-STATUS", IMAPX_CAPABILITY_LIST_STATUS },
 };
 
 struct _capability_info *
@@ -440,13 +405,8 @@ imapx_parse_capability(CamelIMAPXStream *stream, CamelException *ex)
 	cinfo->auth_types = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) g_free, NULL);
 
 	/* FIXME: handle auth types */
-	while ((tok = camel_imapx_stream_token(stream, &token, &len, ex)) != '\n' &&
-	       !camel_exception_is_set (ex)) {
+	while (!camel_exception_is_set (ex) && (tok = camel_imapx_stream_token(stream, &token, &len, ex)) != '\n') {
 		switch (tok) {
-			case ']':
-				/* Put it back so that imapx_untagged() isn't unhappy */
-				camel_imapx_stream_ungettoken(stream, tok, token, len);
-				return cinfo;
 			case 43:
 				token = (guchar *) g_strconcat ((gchar *)token, "+", NULL);
 				free_token = TRUE;
@@ -1321,32 +1281,6 @@ imapx_parse_section(CamelIMAPXStream *is, CamelException *ex)
 	return section;
 }
 
-static guint64
-imapx_parse_modseq(CamelIMAPXStream *is, CamelException *ex)
-{
-	guint64 ret;
-	gint tok;
-	guint len;
-	guchar *token;
-
-	
-	tok = camel_imapx_stream_token(is, &token, &len, ex);
-	if (tok != '(') {
-		camel_exception_set (ex, 1, "fetch: expecting '('");
-		return 0;
-	}
-	ret = camel_imapx_stream_number(is, ex);
-	if (camel_exception_is_set(ex))
-		return 0;
-
-	tok = camel_imapx_stream_token(is, &token, &len, ex);
-	if (tok != ')') {
-		camel_exception_set (ex, 1, "fetch: expecting '('");
-		return 0;
-	}
-	return ret;
-}
-
 void
 imapx_free_fetch(struct _fetch_info *finfo)
 {
@@ -1354,11 +1288,11 @@ imapx_free_fetch(struct _fetch_info *finfo)
 		return;
 
 	if (finfo->body)
-		camel_object_unref((CamelObject *)finfo->body);
+		g_object_unref (finfo->body);
 	if (finfo->text)
-		camel_object_unref((CamelObject *)finfo->text);
+		g_object_unref (finfo->text);
 	if (finfo->header)
-		camel_object_unref((CamelObject *)finfo->header);
+		g_object_unref (finfo->header);
 	if (finfo->minfo)
 		camel_message_info_free(finfo->minfo);
 	if (finfo->cinfo)
@@ -1370,8 +1304,6 @@ imapx_free_fetch(struct _fetch_info *finfo)
 	g_free(finfo);
 }
 
-#include <camel/camel-stream-fs.h>
-
 /* debug, dump one out */
 void
 imapx_dump_fetch(struct _fetch_info *finfo)
@@ -1422,7 +1354,7 @@ imapx_dump_fetch(struct _fetch_info *finfo)
 		camel_stream_printf(sout, "Section: '%s'\n", finfo->section);
 	if (finfo->date)
 		camel_stream_printf(sout, "UID: '%s'\n", finfo->uid);
-	camel_object_unref((CamelObject *)sout);
+	g_object_unref (sout);
 }
 
 struct _fetch_info *
@@ -1479,10 +1411,6 @@ imapx_parse_fetch(CamelIMAPXStream *is, CamelException *ex)
 				finfo->cinfo = imapx_parse_body(is, ex);
 				finfo->got |= FETCH_CINFO;
 				break;
-			case IMAPX_MODSEQ:
-				finfo->modseq = imapx_parse_modseq(is, ex);
-				finfo->got |= FETCH_MODSEQ;
-				break;
 			case IMAPX_BODY:
 				tok = camel_imapx_stream_token(is, &token, &len, ex);
 				camel_imapx_stream_ungettoken(is, tok, token, len);
@@ -1542,12 +1470,7 @@ imapx_parse_status_info (struct _CamelIMAPXStream *is, CamelException *ex)
 	sinfo = g_malloc0 (sizeof(*sinfo));
 
 	/* skip the folder name */
-	camel_imapx_stream_astring (is, &token, ex);
-	if (camel_exception_is_set(ex)) {
-		g_free (sinfo);
-		return NULL;
-	}
-	sinfo->name = camel_utf7_utf8 ((gchar *)token);
+	camel_imapx_stream_token (is, &token, &len, ex);
 
 	tok = camel_imapx_stream_token(is, &token, &len, ex);
 	if (tok != '(') {
@@ -1573,11 +1496,6 @@ imapx_parse_status_info (struct _CamelIMAPXStream *is, CamelException *ex)
 			case IMAPX_UNSEEN:
 				sinfo->unseen = camel_imapx_stream_number (is, ex);
 				break;
-			case IMAPX_HIGHESTMODSEQ:
-				sinfo->highestmodseq = camel_imapx_stream_number (is, ex);
-				break;
-			case IMAPX_NOMODSEQ:
-			break;
 			default:
 				g_free (sinfo);
 				camel_exception_set (ex, 1, "unknown status response");
@@ -1603,7 +1521,7 @@ generate_uids_from_sequence (GPtrArray *uids, guint32 begin_uid, guint32 end_uid
 		g_ptr_array_add (uids, GUINT_TO_POINTER (i));
 }
 
-GPtrArray *
+static GPtrArray *
 imapx_parse_uids (CamelIMAPXStream *is, CamelException *ex)
 {
 	GPtrArray *uids = g_ptr_array_new ();
@@ -1625,7 +1543,7 @@ imapx_parse_uids (CamelIMAPXStream *is, CamelException *ex)
 			generate_uids_from_sequence (uids, uid1, uid2);
 			g_strfreev (seq);
 		} else {
-			guint32 uid = strtoul ((gchar *) splits[i], NULL, 10);
+			guint32 uid = strtoul ((gchar *) token, NULL, 10);
 			g_ptr_array_add (uids, GUINT_TO_POINTER (uid));
 		}
 	}
@@ -1685,7 +1603,6 @@ imapx_parse_status(CamelIMAPXStream *is, CamelException *ex)
 			case IMAPX_ALERT:
 			case IMAPX_PARSE:
 			case IMAPX_TRYCREATE:
-			case IMAPX_CLOSED:
 				break;
 			case IMAPX_APPENDUID:
 				sinfo->u.appenduid.uidvalidity = camel_imapx_stream_number(is, ex);
@@ -1716,12 +1633,6 @@ imapx_parse_status(CamelIMAPXStream *is, CamelException *ex)
 			case IMAPX_UNSEEN:
 				sinfo->u.unseen = camel_imapx_stream_number(is, ex);
 				break;
-			case IMAPX_HIGHESTMODSEQ:
-				sinfo->u.highestmodseq = camel_imapx_stream_number(is, ex);
-				break;
-			case IMAPX_CAPABILITY:
-				sinfo->u.cinfo = imapx_parse_capability(is, ex);
-				break;
 			default:
 				sinfo->condition = IMAPX_UNKNOWN;
 				d(printf("Got unknown response code: %s: ignored\n", token));
@@ -1777,10 +1688,6 @@ imapx_free_status(struct _status_info *sinfo)
 		g_ptr_array_free (sinfo->u.copyuid.uids, FALSE);
 		g_ptr_array_free (sinfo->u.copyuid.copied_uids, FALSE);
 		break;
-	case IMAPX_CAPABILITY:
-		if (sinfo->u.cinfo)
-			imapx_free_capability(sinfo->u.cinfo);
-		break;
 	default:
 		break;
 	}
@@ -1799,7 +1706,6 @@ static struct {
 	{ "\\NOSELECT", CAMEL_FOLDER_NOSELECT },
 	{ "\\MARKED", 1<< 16},
 	{ "\\UNMARKED", 1<< 17},
-	{ "\\SUBSCRIBED", CAMEL_FOLDER_SUBSCRIBED },
 };
 
 struct _list_info *
@@ -1977,7 +1883,6 @@ void imapx_utils_init(void)
 
 		imapx_specials[i] = v;
 	}
-	camel_imapx_set_debug_flags();
 }
 
 guchar imapx_is_mask(const gchar *p)
