@@ -34,19 +34,6 @@
 
 #include <glib/gi18n-lib.h>
 
-#include "camel/camel-data-cache.h"
-#include "camel/camel-debug.h"
-#include "camel/camel-disco-diary.h"
-#include "camel/camel-disco-store.h"
-#include "camel/camel-net-utils.h"
-#include "camel/camel-private.h"
-#include "camel/camel-session.h"
-#include "camel/camel-stream-mem.h"
-#include "camel/camel-string-utils.h"
-#include "camel/camel-tcp-stream-raw.h"
-#include "camel/camel-tcp-stream-ssl.h"
-#include "camel/camel-url.h"
-
 #include "camel-nntp-summary.h"
 #include "camel-nntp-store.h"
 #include "camel-nntp-store-summary.h"
@@ -54,6 +41,11 @@
 #include "camel-nntp-private.h"
 #include "camel-nntp-resp-codes.h"
 
+#ifdef G_OS_WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#endif
+
 #define w(x)
 #define dd(x) (camel_debug("nntp")?(x):0)
 
@@ -62,19 +54,74 @@
 
 #define DUMP_EXTENSIONS
 
-static CamelDiscoStoreClass *parent_class = NULL;
-static CamelServiceClass *service_class = NULL;
-
-/* Returns the class for a CamelNNTPStore */
-#define CNNTPS_CLASS(so) CAMEL_NNTP_STORE_CLASS (CAMEL_OBJECT_GET_CLASS(so))
-#define CF_CLASS(so) CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(so))
-#define CNNTPF_CLASS(so) CAMEL_NNTP_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(so))
+#define CAMEL_NNTP_STORE_GET_PRIVATE(obj) \
+	(G_TYPE_INSTANCE_GET_PRIVATE \
+	((obj), CAMEL_TYPE_NNTP_STORE, CamelNNTPStorePrivate))
 
 static gint camel_nntp_try_authenticate (CamelNNTPStore *store, CamelException *ex);
 
-static void nntp_construct (CamelService *service, CamelSession *session,
-			    CamelProvider *provider, CamelURL *url,
-			    CamelException *ex);
+G_DEFINE_TYPE (CamelNNTPStore, camel_nntp_store, CAMEL_TYPE_DISCO_STORE)
+
+static void
+nntp_store_dispose (GObject *object)
+{
+	CamelNNTPStore *nntp_store = CAMEL_NNTP_STORE (object);
+	CamelDiscoStore *disco_store = CAMEL_DISCO_STORE (object);
+
+	/* Only run this the first time. */
+	if (nntp_store->summary != NULL)
+		camel_service_disconnect (CAMEL_SERVICE (object), TRUE, NULL);
+
+	if (nntp_store->summary != NULL) {
+		camel_store_summary_save (
+			CAMEL_STORE_SUMMARY (nntp_store->summary));
+		g_object_unref (nntp_store->summary);
+		nntp_store->summary = NULL;
+	}
+
+	if (nntp_store->mem != NULL) {
+		g_object_unref (nntp_store->mem);
+		nntp_store->mem = NULL;
+	}
+
+	if (nntp_store->stream != NULL) {
+		g_object_unref (nntp_store->stream);
+		nntp_store->stream = NULL;
+	}
+
+	if (nntp_store->cache != NULL) {
+		g_object_unref (nntp_store->cache);
+		nntp_store->cache = NULL;
+	}
+
+	if (disco_store->diary != NULL) {
+		g_object_unref (disco_store->diary);
+		disco_store->diary = NULL;
+	}
+
+	/* Chain up to parent's dispose() method. */
+	G_OBJECT_CLASS (camel_nntp_store_parent_class)->dispose (object);
+}
+
+static void
+nntp_store_finalize (GObject *object)
+{
+	CamelNNTPStore *nntp_store = CAMEL_NNTP_STORE (object);
+	struct _xover_header *xover, *xn;
+
+	g_free (nntp_store->base_url);
+	g_free (nntp_store->storage_path);
+
+	xover = nntp_store->xover;
+	while (xover) {
+		xn = xover->next;
+		g_free(xover);
+		xover = xn;
+	}
+
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (camel_nntp_store_parent_class)->finalize (object);
+}
 
 static gboolean
 nntp_can_work_offline(CamelDiscoStore *store)
@@ -168,7 +215,7 @@ connect_to_server (CamelService *service, struct addrinfo *ai, gint ssl_mode, Ca
 	guint len;
 	gchar *path;
 
-	CAMEL_SERVICE_REC_LOCK(store, connect_lock);
+	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	if (ssl_mode != MODE_CLEAR) {
 #ifdef HAVE_SSL
@@ -178,10 +225,10 @@ connect_to_server (CamelService *service, struct addrinfo *ai, gint ssl_mode, Ca
 			tcp_stream = camel_tcp_stream_ssl_new (service->session, service->url->host, SSL_PORT_FLAGS);
 		}
 #else
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-				      _("Could not connect to %s: %s"),
-				      service->url->host, _("SSL unavailable"));
-
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+			_("Could not connect to %s: %s"),
+			service->url->host, _("SSL unavailable"));
 		goto fail;
 #endif /* HAVE_SSL */
 	} else {
@@ -190,21 +237,21 @@ connect_to_server (CamelService *service, struct addrinfo *ai, gint ssl_mode, Ca
 
 	if (camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, ai) == -1) {
 		if (errno == EINTR)
-			camel_exception_set (ex, CAMEL_EXCEPTION_USER_CANCEL,
-					     _("Connection canceled"));
+			camel_exception_set (
+				ex, CAMEL_EXCEPTION_USER_CANCEL,
+				_("Connection canceled"));
 		else
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
-					      _("Could not connect to %s: %s"),
-					      service->url->host,
-					      g_strerror (errno));
-
-		camel_object_unref (tcp_stream);
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_SERVICE_UNAVAILABLE,
+				_("Could not connect to %s: %s"),
+				service->url->host, g_strerror (errno));
+		g_object_unref (tcp_stream);
 
 		goto fail;
 	}
 
 	store->stream = (CamelNNTPStream *) camel_nntp_stream_new (tcp_stream);
-	camel_object_unref (tcp_stream);
+	g_object_unref (tcp_stream);
 
 	/* Read the greeting, if any. */
 	if (camel_nntp_stream_line (store->stream, &buf, &len) == -1) {
@@ -216,7 +263,7 @@ connect_to_server (CamelService *service, struct addrinfo *ai, gint ssl_mode, Ca
 					      _("Could not read greeting from %s: %s"),
 					      service->url->host, g_strerror (errno));
 
-		camel_object_unref (store->stream);
+		g_object_unref (store->stream);
 		store->stream = NULL;
 
 		goto fail;
@@ -228,7 +275,7 @@ connect_to_server (CamelService *service, struct addrinfo *ai, gint ssl_mode, Ca
 				      _("NNTP server %s returned error code %d: %s"),
 				      service->url->host, len, buf);
 
-		camel_object_unref (store->stream);
+		g_object_unref (store->stream);
 		store->stream = NULL;
 
 		goto fail;
@@ -260,7 +307,7 @@ connect_to_server (CamelService *service, struct addrinfo *ai, gint ssl_mode, Ca
 	store->current_folder = NULL;
 
  fail:
-	CAMEL_SERVICE_REC_UNLOCK(store, connect_lock);
+	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 	return retval;
 }
 
@@ -335,7 +382,7 @@ nntp_connect_offline (CamelService *service, CamelException *ex)
 
 	/* setup store-wide cache */
 	if (nntp_store->cache == NULL) {
-		nntp_store->cache = camel_data_cache_new (nntp_store->storage_path, 0, ex);
+		nntp_store->cache = camel_data_cache_new (nntp_store->storage_path, ex);
 		if (nntp_store->cache == NULL)
 			return FALSE;
 
@@ -361,9 +408,12 @@ static gboolean
 nntp_disconnect_online (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelNNTPStore *store = CAMEL_NNTP_STORE (service);
+	CamelServiceClass *service_class;
 	gchar *line;
 
-	CAMEL_SERVICE_REC_LOCK(store, connect_lock);
+	service_class = CAMEL_SERVICE_GET_CLASS (service);
+
+	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	if (clean) {
 		camel_nntp_raw_command (store, ex, &line, "quit");
@@ -371,16 +421,16 @@ nntp_disconnect_online (CamelService *service, gboolean clean, CamelException *e
 	}
 
 	if (!service_class->disconnect (service, clean, ex)) {
-		CAMEL_SERVICE_REC_UNLOCK(store, connect_lock);
+		camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 		return FALSE;
 	}
 
-	camel_object_unref (store->stream);
+	g_object_unref (store->stream);
 	store->stream = NULL;
 	g_free(store->current_folder);
 	store->current_folder = NULL;
 
-	CAMEL_SERVICE_REC_UNLOCK(store, connect_lock);
+	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return TRUE;
 }
@@ -389,12 +439,15 @@ static gboolean
 nntp_disconnect_offline (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelDiscoStore *disco = CAMEL_DISCO_STORE(service);
+	CamelServiceClass *service_class;
 
+	/* Chain up to parent's disconnect() method. */
+	service_class = CAMEL_SERVICE_CLASS (camel_nntp_store_parent_class);
 	if (!service_class->disconnect (service, clean, ex))
 		return FALSE;
 
 	if (disco->diary) {
-		camel_object_unref (disco->diary);
+		g_object_unref (disco->diary);
 		disco->diary = NULL;
 	}
 
@@ -425,11 +478,11 @@ nntp_get_folder(CamelStore *store, const gchar *folder_name, guint32 flags, Came
 	CamelNNTPStore *nntp_store = CAMEL_NNTP_STORE (store);
 	CamelFolder *folder;
 
-	CAMEL_SERVICE_REC_LOCK(nntp_store, connect_lock);
+	camel_service_lock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	folder = camel_nntp_folder_new(store, folder_name, ex);
 
-	CAMEL_SERVICE_REC_UNLOCK(nntp_store, connect_lock);
+	camel_service_unlock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return folder;
 }
@@ -614,18 +667,18 @@ nntp_store_get_subscribed_folder_info (CamelNNTPStore *store, const gchar *top,
 				if (folder) {
 					CamelFolderChangeInfo *changes = NULL;
 
-					CAMEL_SERVICE_REC_LOCK(store, connect_lock);
+					camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 					camel_nntp_command(store, ex, folder, &line, NULL);
 					if (camel_folder_change_info_changed(folder->changes)) {
 						changes = folder->changes;
 						folder->changes = camel_folder_change_info_new();
 					}
-					CAMEL_SERVICE_REC_UNLOCK(store, connect_lock);
+					camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 					if (changes) {
-						camel_object_trigger_event((CamelObject *) folder, "folder_changed", changes);
+						camel_folder_changed (CAMEL_FOLDER (folder), changes);
 						camel_folder_change_info_free(changes);
 					}
-					camel_object_unref(folder);
+					g_object_unref (folder);
 				}
 				camel_exception_clear(ex);
 			}
@@ -643,6 +696,91 @@ nntp_store_get_subscribed_folder_info (CamelNNTPStore *store, const gchar *top,
 	return first;
 }
 
+/* returns new root */
+static CamelFolderInfo *
+nntp_push_to_hierarchy (CamelURL *base_url, CamelFolderInfo *root, CamelFolderInfo *pfi, GHashTable *known)
+{
+	CamelFolderInfo *fi, *last = NULL, *kfi;
+	gchar *name, *dot;
+
+	g_return_val_if_fail (pfi != NULL, root);
+	g_return_val_if_fail (known != NULL, root);
+
+	name = pfi->full_name;
+	g_return_val_if_fail (name != NULL, root);
+
+	while (dot = strchr (name, '.'), dot) {
+		*dot = '\0';
+
+		kfi = g_hash_table_lookup (known, pfi->full_name);
+		if (!kfi) {
+			gchar *path;
+			CamelURL *url;
+
+			fi = camel_folder_info_new ();
+			fi->full_name = g_strdup (pfi->full_name);
+			fi->name = g_strdup (name);
+
+			fi->unread = 0;
+			fi->total = 0;
+			fi->flags = CAMEL_FOLDER_NOSELECT | CAMEL_FOLDER_CHILDREN;
+			path = alloca (strlen (fi->full_name) + 2);
+			sprintf (path, "/%s", fi->full_name);
+			url = camel_url_new_with_base (base_url, path);
+			fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+			camel_url_free (url);
+
+			g_hash_table_insert (known, fi->full_name, fi);
+			if (last) {
+				if (!last->child) {
+					last->child = fi;
+					fi->parent = last;
+				} else {
+					kfi = last->child;
+					while (kfi->next)
+						kfi = kfi->next;
+					kfi->next = fi;
+					fi->parent = last;
+				}
+			}
+			last = fi;
+			if (!root)
+				root = fi;
+		} else {
+			last = kfi;
+		}
+
+		*dot = '.';
+		name = dot + 1;
+	}
+
+	g_free (pfi->name);
+	pfi->name = g_strdup (name);
+
+	if (!root) {
+		root = pfi;
+	} else if (!last) {
+		kfi = root;
+		while (kfi->next)
+			kfi = kfi->next;
+		kfi->next = pfi;
+		pfi->parent = kfi->parent;
+	} else {
+		if (!last->child) {
+			last->child = pfi;
+			pfi->parent = last;
+		} else {
+			kfi = last->child;
+			while (kfi->next)
+				kfi = kfi->next;
+			kfi->next = pfi;
+			pfi->parent = last;
+		}
+	}
+
+	return root;
+}
+
 /*
  * get folder info, using the information in our StoreSummary
  */
@@ -652,17 +790,21 @@ nntp_store_get_cached_folder_info (CamelNNTPStore *store, const gchar *orig_top,
 	gint i;
 	gint subscribed_or_flag = (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED) ? 0 : 1,
 	    root_or_flag = (orig_top == NULL || orig_top[0] == '\0') ? 1 : 0,
-	    recursive_flag = flags & CAMEL_STORE_FOLDER_INFO_RECURSIVE;
+	    recursive_flag = flags & CAMEL_STORE_FOLDER_INFO_RECURSIVE,
+	    is_folder_list = flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIPTION_LIST;
 	CamelStoreInfo *si;
-	CamelFolderInfo *first = NULL, *last = NULL, *fi = NULL, *l;
+	CamelFolderInfo *first = NULL, *last = NULL, *fi = NULL;
+	GHashTable *known; /* folder name to folder info */
 	gchar *tmpname;
 	gchar *top = g_strconcat(orig_top?orig_top:"", ".", NULL);
 	gint toplen = strlen(top);
 
+	known = g_hash_table_new (g_str_hash, g_str_equal);
+
 	for (i = 0; (si = camel_store_summary_index ((CamelStoreSummary *) store->summary, i)); i++) {
 		if ((subscribed_or_flag || (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED))
 		    && (root_or_flag || strncmp (si->path, top, toplen) == 0)) {
-			if (recursive_flag || strchr (si->path + toplen, '.') == NULL) {
+			if (recursive_flag || is_folder_list || strchr (si->path + toplen, '.') == NULL) {
 				/* add the item */
 				fi = nntp_folder_info_from_store_info(store, FALSE, si);
 				if (!fi)
@@ -695,24 +837,24 @@ nntp_store_get_cached_folder_info (CamelNNTPStore *store, const gchar *orig_top,
 				}
 			}
 
-			for (l = first; l; l = l->next) {
-				if (l->full_name && fi->full_name && g_str_equal (l->full_name, fi->full_name)) {
-					/* it's there already, do not add it twice */
-					camel_folder_info_free (fi);
-					break;
-				}
-			}
-
-			if (l) {
+			if (fi->full_name && g_hash_table_lookup (known, fi->full_name)) {
 				/* a duplicate has been found above */
+				camel_folder_info_free (fi);
 				continue;
 			}
 
-			if (last)
-				last->next = fi;
-			else
-				first = fi;
-			last = fi;
+			g_hash_table_insert (known, fi->full_name, fi);
+
+			if (is_folder_list) {
+				/* create a folder hierarchy rather than a flat list */
+				first = nntp_push_to_hierarchy (((CamelService *)store)->url, first, fi, known);
+			} else {
+				if (last)
+					last->next = fi;
+				else
+					first = fi;
+				last = fi;
+			}
 		} else if (subscribed_or_flag && first) {
 			/* we have already added subitems, but this item is no longer a subitem */
 			camel_store_summary_info_free((CamelStoreSummary *)store->summary, si);
@@ -721,6 +863,7 @@ nntp_store_get_cached_folder_info (CamelNNTPStore *store, const gchar *orig_top,
 		camel_store_summary_info_free((CamelStoreSummary *)store->summary, si);
 	}
 
+	g_hash_table_destroy (known);
 	g_free(top);
 	return first;
 }
@@ -773,7 +916,7 @@ nntp_store_get_folder_info_all(CamelNNTPStore *nntp_store, const gchar *top, gui
 	gint ret = -1;
 	CamelFolderInfo *fi = NULL;
 
-	CAMEL_SERVICE_REC_LOCK(nntp_store, connect_lock);
+	camel_service_lock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	if (top == NULL)
 		top = "";
@@ -815,8 +958,9 @@ nntp_store_get_folder_info_all(CamelNNTPStore *nntp_store, const gchar *top, gui
 			if (ret == -1)
 				goto error;
 			else if (ret != 215) {
-				camel_exception_setv (ex, CAMEL_EXCEPTION_SERVICE_INVALID,
-						      _("Error retrieving newsgroups:\n\n%s"), line);
+				camel_exception_setv (
+					ex, CAMEL_EXCEPTION_SERVICE_INVALID,
+					_("Error retrieving newsgroups:\n\n%s"), line);
 				goto error;
 			}
 
@@ -843,7 +987,7 @@ nntp_store_get_folder_info_all(CamelNNTPStore *nntp_store, const gchar *top, gui
 
 	fi = nntp_store_get_cached_folder_info (nntp_store, top, flags, ex);
  error:
-	CAMEL_SERVICE_REC_UNLOCK(nntp_store, connect_lock);
+	camel_service_unlock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return fi;
 }
@@ -882,7 +1026,7 @@ nntp_get_folder_info_offline(CamelStore *store, const gchar *top, guint32 flags,
 }
 
 static gboolean
-nntp_store_folder_subscribed (CamelStore *store, const gchar *folder_name)
+nntp_store_folder_is_subscribed (CamelStore *store, const gchar *folder_name)
 {
 	CamelNNTPStore *nntp_store = CAMEL_NNTP_STORE (store);
 	CamelStoreInfo *si;
@@ -897,21 +1041,25 @@ nntp_store_folder_subscribed (CamelStore *store, const gchar *folder_name)
 	return truth;
 }
 
-static void
+static gboolean
 nntp_store_subscribe_folder (CamelStore *store, const gchar *folder_name,
 			     CamelException *ex)
 {
 	CamelNNTPStore *nntp_store = CAMEL_NNTP_STORE(store);
 	CamelStoreInfo *si;
 	CamelFolderInfo *fi;
+	gboolean success = TRUE;
 
-	CAMEL_SERVICE_REC_LOCK(nntp_store, connect_lock);
+	camel_service_lock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	si = camel_store_summary_path(CAMEL_STORE_SUMMARY(nntp_store->summary), folder_name);
 	if (!si) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID,
-				      _("You cannot subscribe to this newsgroup:\n\n"
-					"No such newsgroup. The selected item is a probably a parent folder."));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_FOLDER_INVALID,
+			_("You cannot subscribe to this newsgroup:\n\n"
+			  "No such newsgroup. The selected item is a "
+			  "probably a parent folder."));
+		success = FALSE;
 	} else {
 		if (!(si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED)) {
 			si->flags |= CAMEL_STORE_INFO_FOLDER_SUBSCRIBED;
@@ -919,45 +1067,53 @@ nntp_store_subscribe_folder (CamelStore *store, const gchar *folder_name,
 			fi->flags |= CAMEL_FOLDER_NOINFERIORS | CAMEL_FOLDER_NOCHILDREN;
 			camel_store_summary_touch ((CamelStoreSummary *) nntp_store->summary);
 			camel_store_summary_save ((CamelStoreSummary *) nntp_store->summary);
-			CAMEL_SERVICE_REC_UNLOCK(nntp_store, connect_lock);
-			camel_object_trigger_event ((CamelObject *) nntp_store, "folder_subscribed", fi);
+			camel_service_unlock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+			camel_store_folder_subscribed (CAMEL_STORE (nntp_store), fi);
 			camel_folder_info_free (fi);
-			return;
+			return TRUE;
 		}
 	}
 
-	CAMEL_SERVICE_REC_UNLOCK(nntp_store, connect_lock);
+	camel_service_unlock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+
+	return success;
 }
 
-static void
+static gboolean
 nntp_store_unsubscribe_folder (CamelStore *store, const gchar *folder_name,
 			       CamelException *ex)
 {
 	CamelNNTPStore *nntp_store = CAMEL_NNTP_STORE(store);
 	CamelFolderInfo *fi;
 	CamelStoreInfo *fitem;
-	CAMEL_SERVICE_REC_LOCK(nntp_store, connect_lock);
+	gboolean success = TRUE;
+
+	camel_service_lock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	fitem = camel_store_summary_path(CAMEL_STORE_SUMMARY(nntp_store->summary), folder_name);
 
 	if (!fitem) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID,
-				      _("You cannot unsubscribe to this newsgroup:\n\n"
-					"newsgroup does not exist!"));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_FOLDER_INVALID,
+			_("You cannot unsubscribe to this newsgroup:\n\n"
+			  "newsgroup does not exist!"));
+		success = FALSE;
 	} else {
 		if (fitem->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED) {
 			fitem->flags &= ~CAMEL_STORE_INFO_FOLDER_SUBSCRIBED;
 			fi = nntp_folder_info_from_store_info (nntp_store, nntp_store->do_short_folder_notation, fitem);
 			camel_store_summary_touch ((CamelStoreSummary *) nntp_store->summary);
 			camel_store_summary_save ((CamelStoreSummary *) nntp_store->summary);
-			CAMEL_SERVICE_REC_UNLOCK(nntp_store, connect_lock);
-			camel_object_trigger_event ((CamelObject *) nntp_store, "folder_unsubscribed", fi);
+			camel_service_unlock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+			camel_store_folder_unsubscribed (CAMEL_STORE (nntp_store), fi);
 			camel_folder_info_free (fi);
-			return;
+			return TRUE;
 		}
 	}
 
-	CAMEL_SERVICE_REC_UNLOCK(nntp_store, connect_lock);
+	camel_service_unlock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+
+	return success;
 }
 
 /* stubs for various folder operations we're not implementing */
@@ -966,69 +1122,35 @@ static CamelFolderInfo *
 nntp_create_folder (CamelStore *store, const gchar *parent_name,
                     const gchar *folder_name, CamelException *ex)
 {
-	camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID,
-		    _("You cannot create a folder in a News store: subscribe instead."));
+	camel_exception_setv (
+		ex, CAMEL_EXCEPTION_FOLDER_INVALID,
+		_("You cannot create a folder in a News store: "
+		  "subscribe instead."));
+
 	return NULL;
 }
 
-static void
+static gboolean
 nntp_rename_folder (CamelStore *store, const gchar *old_name, const gchar *new_name_in, CamelException *ex)
 {
-	camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID,
-		  _("You cannot rename a folder in a News store."));
+	camel_exception_setv (
+		ex, CAMEL_EXCEPTION_FOLDER_INVALID,
+		_("You cannot rename a folder in a News store."));
+
+	return FALSE;
 }
 
-static void
+static gboolean
 nntp_delete_folder (CamelStore *store, const gchar *folder_name, CamelException *ex)
 {
 	nntp_store_unsubscribe_folder (store, folder_name, ex);
-	camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID,
-		  _("You cannot remove a folder in a News store: unsubscribe instead."));
-	return;
-}
 
-static void
-nntp_store_finalize (CamelObject *object)
-{
-	/* call base finalize */
-	CamelNNTPStore *nntp_store = CAMEL_NNTP_STORE (object);
-	CamelDiscoStore *disco_store = (CamelDiscoStore *) nntp_store;
-	struct _CamelNNTPStorePrivate *p = nntp_store->priv;
-	struct _xover_header *xover, *xn;
+	camel_exception_setv (
+		ex, CAMEL_EXCEPTION_FOLDER_INVALID,
+		_("You cannot remove a folder in a News store: "
+		  "unsubscribe instead."));
 
-	camel_service_disconnect ((CamelService *)object, TRUE, NULL);
-
-	if (nntp_store->summary) {
-		camel_store_summary_save ((CamelStoreSummary *) nntp_store->summary);
-		camel_object_unref (nntp_store->summary);
-	}
-
-	camel_object_unref (nntp_store->mem);
-	nntp_store->mem = NULL;
-	if (nntp_store->stream)
-		camel_object_unref (nntp_store->stream);
-
-	if (nntp_store->base_url)
-		g_free (nntp_store->base_url);
-	if (nntp_store->storage_path)
-		g_free (nntp_store->storage_path);
-
-	xover = nntp_store->xover;
-	while (xover) {
-		xn = xover->next;
-		g_free(xover);
-		xover = xn;
-	}
-
-	if (nntp_store->cache)
-		camel_object_unref(nntp_store->cache);
-
-	if (disco_store->diary) {
-		camel_object_unref (disco_store->diary);
-		disco_store->diary = NULL;
-	}
-
-	g_free(p);
+	return FALSE;
 }
 
 static gboolean
@@ -1038,65 +1160,26 @@ nntp_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, CamelExceptio
 	return TRUE;
 }
 
-static void
-nntp_store_class_init (CamelNNTPStoreClass *camel_nntp_store_class)
-{
-	CamelDiscoStoreClass *camel_disco_store_class = CAMEL_DISCO_STORE_CLASS (camel_nntp_store_class);
-	CamelStoreClass *camel_store_class = CAMEL_STORE_CLASS (camel_nntp_store_class);
-	CamelServiceClass *camel_service_class = CAMEL_SERVICE_CLASS (camel_nntp_store_class);
-
-	parent_class = CAMEL_DISCO_STORE_CLASS (camel_type_get_global_classfuncs (camel_disco_store_get_type ()));
-	service_class = CAMEL_SERVICE_CLASS (camel_type_get_global_classfuncs (camel_service_get_type ()));
-
-	/* virtual method overload */
-	camel_service_class->construct = nntp_construct;
-	camel_service_class->query_auth_types = nntp_store_query_auth_types;
-	camel_service_class->get_name = nntp_store_get_name;
-
-	camel_disco_store_class->can_work_offline = nntp_can_work_offline;
-	camel_disco_store_class->connect_online = nntp_connect_online;
-	camel_disco_store_class->connect_offline = nntp_connect_offline;
-	camel_disco_store_class->disconnect_online = nntp_disconnect_online;
-	camel_disco_store_class->disconnect_offline = nntp_disconnect_offline;
-	camel_disco_store_class->get_folder_online = nntp_get_folder;
-	camel_disco_store_class->get_folder_resyncing = nntp_get_folder;
-	camel_disco_store_class->get_folder_offline = nntp_get_folder;
-
-	camel_disco_store_class->get_folder_info_online = nntp_get_folder_info_online;
-	camel_disco_store_class->get_folder_info_resyncing = nntp_get_folder_info_online;
-	camel_disco_store_class->get_folder_info_offline = nntp_get_folder_info_offline;
-
-	camel_store_class->free_folder_info = camel_store_free_folder_info_full;
-
-	camel_store_class->folder_subscribed = nntp_store_folder_subscribed;
-	camel_store_class->subscribe_folder = nntp_store_subscribe_folder;
-	camel_store_class->unsubscribe_folder = nntp_store_unsubscribe_folder;
-
-	camel_store_class->create_folder = nntp_create_folder;
-	camel_store_class->delete_folder = nntp_delete_folder;
-	camel_store_class->rename_folder = nntp_rename_folder;
-	camel_store_class->can_refresh_folder = nntp_can_refresh_folder;
-}
-
 /* construction function in which we set some basic store properties */
-static void
+static gboolean
 nntp_construct (CamelService *service, CamelSession *session,
 		CamelProvider *provider, CamelURL *url,
 		CamelException *ex)
 {
+	CamelServiceClass *service_class;
 	CamelNNTPStore *nntp_store = CAMEL_NNTP_STORE(service);
 	CamelURL *summary_url;
 	gchar *tmp;
 
 	/* construct the parent first */
-	CAMEL_SERVICE_CLASS (parent_class)->construct (service, session, provider, url, ex);
-	if (camel_exception_is_set (ex))
-		return;
+	service_class = CAMEL_SERVICE_CLASS (camel_nntp_store_parent_class);
+	if (!service_class->construct (service, session, provider, url, ex))
+		return FALSE;
 
 	/* find out the storage path, base url */
 	nntp_store->storage_path = camel_session_get_storage_path (session, service, ex);
 	if (!nntp_store->storage_path)
-		return;
+		return FALSE;
 
 	/* FIXME */
 	nntp_store->base_url = camel_url_to_string (service->url, (CAMEL_URL_HIDE_PASSWORD |
@@ -1124,47 +1207,70 @@ nntp_construct (CamelService *service, CamelSession *session,
 		nntp_store->folder_hierarchy_relative = FALSE;
 
 	/* setup store-wide cache */
-	nntp_store->cache = camel_data_cache_new(nntp_store->storage_path, 0, ex);
+	nntp_store->cache = camel_data_cache_new(nntp_store->storage_path, ex);
 	if (nntp_store->cache == NULL)
-		return;
+		return FALSE;
 
 	/* Default cache expiry - 2 weeks old, or not visited in 5 days */
 	camel_data_cache_set_expire_age(nntp_store->cache, 60*60*24*14);
 	camel_data_cache_set_expire_access(nntp_store->cache, 60*60*24*5);
+
+	return TRUE;
 }
 
 static void
-nntp_store_init (gpointer object, gpointer klass)
+camel_nntp_store_class_init (CamelNNTPStoreClass *class)
 {
-	CamelNNTPStore *nntp_store = CAMEL_NNTP_STORE(object);
-	CamelStore *store = CAMEL_STORE (object);
-	struct _CamelNNTPStorePrivate *p;
+	GObjectClass *object_class;
+	CamelServiceClass *service_class;
+	CamelStoreClass *store_class;
+	CamelDiscoStoreClass *disco_store_class;
+
+	g_type_class_add_private (class, sizeof (CamelNNTPStorePrivate));
+
+	object_class = G_OBJECT_CLASS (class);
+	object_class->dispose = nntp_store_dispose;
+	object_class->finalize = nntp_store_finalize;
+
+	service_class = CAMEL_SERVICE_CLASS (class);
+	service_class->construct = nntp_construct;
+	service_class->query_auth_types = nntp_store_query_auth_types;
+	service_class->get_name = nntp_store_get_name;
+
+	store_class = CAMEL_STORE_CLASS (class);
+	store_class->free_folder_info = camel_store_free_folder_info_full;
+	store_class->folder_is_subscribed = nntp_store_folder_is_subscribed;
+	store_class->subscribe_folder = nntp_store_subscribe_folder;
+	store_class->unsubscribe_folder = nntp_store_unsubscribe_folder;
+	store_class->create_folder = nntp_create_folder;
+	store_class->delete_folder = nntp_delete_folder;
+	store_class->rename_folder = nntp_rename_folder;
+	store_class->can_refresh_folder = nntp_can_refresh_folder;
+
+	disco_store_class = CAMEL_DISCO_STORE_CLASS (class);
+	disco_store_class->can_work_offline = nntp_can_work_offline;
+	disco_store_class->connect_online = nntp_connect_online;
+	disco_store_class->connect_offline = nntp_connect_offline;
+	disco_store_class->disconnect_online = nntp_disconnect_online;
+	disco_store_class->disconnect_offline = nntp_disconnect_offline;
+	disco_store_class->get_folder_online = nntp_get_folder;
+	disco_store_class->get_folder_resyncing = nntp_get_folder;
+	disco_store_class->get_folder_offline = nntp_get_folder;
+	disco_store_class->get_folder_info_online = nntp_get_folder_info_online;
+	disco_store_class->get_folder_info_resyncing = nntp_get_folder_info_online;
+	disco_store_class->get_folder_info_offline = nntp_get_folder_info_offline;
+}
+
+static void
+camel_nntp_store_init (CamelNNTPStore *nntp_store)
+{
+	CamelStore *store = CAMEL_STORE (nntp_store);
 
 	store->flags = CAMEL_STORE_SUBSCRIPTIONS;
 
 	nntp_store->mem = (CamelStreamMem *)camel_stream_mem_new();
 
-	p = nntp_store->priv = g_malloc0(sizeof(*p));
-}
-
-CamelType
-camel_nntp_store_get_type (void)
-{
-	static CamelType camel_nntp_store_type = CAMEL_INVALID_TYPE;
-
-	if (camel_nntp_store_type == CAMEL_INVALID_TYPE) {
-		camel_nntp_store_type =
-			camel_type_register (CAMEL_DISCO_STORE_TYPE,
-					     "CamelNNTPStore",
-					     sizeof (CamelNNTPStore),
-					     sizeof (CamelNNTPStoreClass),
-					     (CamelObjectClassInitFunc) nntp_store_class_init,
-					     NULL,
-					     (CamelObjectInitFunc) nntp_store_init,
-					     (CamelObjectFinalizeFunc) nntp_store_finalize);
-	}
-
-	return camel_nntp_store_type;
+	nntp_store->priv = CAMEL_NNTP_STORE_GET_PRIVATE (nntp_store);
 }
 
 static gint
@@ -1176,8 +1282,9 @@ camel_nntp_try_authenticate (CamelNNTPStore *store, CamelException *ex)
 	gchar *line = NULL;
 
 	if (!service->url->user) {
-		camel_exception_setv(ex, CAMEL_EXCEPTION_INVALID_PARAM,
-				     _("Authentication requested but no username provided"));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_INVALID_PARAM,
+			_("Authentication requested but no username provided"));
 		return -1;
 	}
 
@@ -1200,12 +1307,14 @@ camel_nntp_try_authenticate (CamelNNTPStore *store, CamelException *ex)
 
 		service->url->passwd =
 			camel_session_get_password (session, service, NULL,
-						    prompt, "password", CAMEL_SESSION_PASSWORD_SECRET, ex);
+						    prompt, "password", CAMEL_SESSION_PASSWORD_SECRET | (store->password_reprompt ? CAMEL_SESSION_PASSWORD_REPROMPT : 0), ex);
 		g_free(prompt);
 		g_free(base);
 
 		if (!service->url->passwd)
 			return -1;
+
+		store->password_reprompt = FALSE;
 	}
 
 	/* now, send auth info (currently, only authinfo user/pass is supported) */
@@ -1219,8 +1328,8 @@ camel_nntp_try_authenticate (CamelNNTPStore *store, CamelException *ex)
 			    camel_exception_get_id (ex) == CAMEL_EXCEPTION_SERVICE_UNAVAILABLE)
 				return ret;
 
-			/* Need to forget the password here since we have no context on it */
-			camel_session_forget_password(session, service, NULL, "password", ex);
+			/* To force password reprompt */
+			store->password_reprompt = TRUE;
 			g_free (service->url->passwd);
 			service->url->passwd = NULL;
 			goto retry;
@@ -1235,6 +1344,7 @@ camel_nntp_try_authenticate (CamelNNTPStore *store, CamelException *ex)
 gint
 camel_nntp_raw_commandv (CamelNNTPStore *store, CamelException *ex, gchar **line, const gchar *fmt, va_list ap)
 {
+	GByteArray *byte_array;
 	const guchar *p, *ps;
 	guchar c;
 	gchar *s;
@@ -1287,15 +1397,16 @@ camel_nntp_raw_commandv (CamelNNTPStore *store, CamelException *ex, gchar **line
 	}
 
 	camel_stream_write ((CamelStream *) store->mem, (const gchar *) ps, p-ps-1);
-	dd(printf("NNTP_COMMAND: '%.*s'\n", (gint)store->mem->buffer->len, store->mem->buffer->data));
 	camel_stream_write ((CamelStream *) store->mem, "\r\n", 2);
 
-	if (camel_stream_write((CamelStream *) store->stream, (const gchar *) store->mem->buffer->data, store->mem->buffer->len) == -1)
+	byte_array = camel_stream_mem_get_byte_array (store->mem);
+
+	if (camel_stream_write((CamelStream *) store->stream, (const gchar *) byte_array->data, byte_array->len) == -1)
 		goto ioerror;
 
 	/* FIXME: hack */
 	camel_stream_reset ((CamelStream *) store->mem);
-	g_byte_array_set_size (store->mem->buffer, 0);
+	g_byte_array_set_size (byte_array, 0);
 
 	if (camel_nntp_stream_line (store->stream, (guchar **) line, &u) == -1)
 		goto ioerror;
@@ -1310,9 +1421,13 @@ camel_nntp_raw_commandv (CamelNNTPStore *store, CamelException *ex, gchar **line
 
 ioerror:
 	if (errno == EINTR)
-		camel_exception_setv(ex, CAMEL_EXCEPTION_USER_CANCEL, _("Canceled."));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_USER_CANCEL,
+			_("User cancelled the operation"));
 	else
-		camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM, _("NNTP Command failed: %s"), g_strerror(errno));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SYSTEM,
+			_("NNTP Command failed: %s"), g_strerror(errno));
 	return -1;
 }
 
@@ -1359,17 +1474,22 @@ camel_nntp_raw_command_auth(CamelNNTPStore *store, CamelException *ex, gchar **l
 gint
 camel_nntp_command (CamelNNTPStore *store, CamelException *ex, CamelNNTPFolder *folder, gchar **line, const gchar *fmt, ...)
 {
+	const gchar *full_name = NULL;
 	const guchar *p;
 	va_list ap;
 	gint ret, retry;
 	guint u;
 
 	if (((CamelDiscoStore *)store)->status == CAMEL_DISCO_STORE_OFFLINE) {
-		camel_exception_setv(ex, CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
-				     _("Not connected."));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SERVICE_NOT_CONNECTED,
+			_("Not connected."));
 		return -1;
 	}
 
+	if (folder != NULL)
+		full_name = camel_folder_get_full_name (CAMEL_FOLDER (folder));
+
 	retry = 0;
 	do {
 		retry ++;
@@ -1378,7 +1498,7 @@ camel_nntp_command (CamelNNTPStore *store, CamelException *ex, CamelNNTPFolder *
 		    && !camel_service_connect (CAMEL_SERVICE (store), ex))
 			return -1;
 
-		/* Check for unprocessed data, ! */
+		/* Check for unprocessed data, !*/
 		if (store->stream->mode == CAMEL_NNTP_STREAM_DATA) {
 			g_warning("Unprocessed data left in stream, flushing");
 			while (camel_nntp_stream_getd(store->stream, (guchar **)&p, &u) > 0)
@@ -1387,11 +1507,11 @@ camel_nntp_command (CamelNNTPStore *store, CamelException *ex, CamelNNTPFolder *
 		camel_nntp_stream_set_mode(store->stream, CAMEL_NNTP_STREAM_LINE);
 
 		if (folder != NULL
-		    && (store->current_folder == NULL || strcmp(store->current_folder, ((CamelFolder *)folder)->full_name) != 0)) {
-			ret = camel_nntp_raw_command_auth(store, ex, line, "group %s", ((CamelFolder *)folder)->full_name);
+		    && (store->current_folder == NULL || strcmp(store->current_folder, full_name) != 0)) {
+			ret = camel_nntp_raw_command_auth(store, ex, line, "group %s", full_name);
 			if (ret == 211) {
 				g_free(store->current_folder);
-				store->current_folder = g_strdup(((CamelFolder *)folder)->full_name);
+				store->current_folder = g_strdup (full_name);
 				camel_nntp_folder_selected(folder, *line, ex);
 				if (camel_exception_is_set(ex)) {
 					ret = -1;
@@ -1418,8 +1538,9 @@ camel_nntp_command (CamelNNTPStore *store, CamelException *ex, CamelNNTPFolder *
 			ret = -1;
 			continue;
 		case 411:	/* no such group */
-			camel_exception_setv(ex, CAMEL_EXCEPTION_FOLDER_INVALID,
-					     _("No such folder: %s"), line);
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_FOLDER_INVALID,
+				_("No such folder: %s"), line);
 			return -1;
 		case 400:	/* service discontinued */
 		case 401:	/* wrong client state - this should quit but this is what the old code did */
